#ifndef __FOLD_H__
#define __FOLD_H__

#include <vector>
#include <iostream>

#include "QAP/QAPSum.hpp"
#include "utility/Format.hpp"
#include "Result.hpp"

class Learner
{
public:
  virtual ~Learner();

  virtual void reset() = 0;

  virtual void train(const std::vector<QAPSum> &qap, const std::vector<Permutation> &truth) = 0;
  virtual Permutation predict(const QAPSum &qap, int *nvisited) = 0;

  virtual Learner *clone() const = 0;
};

class Fold
{
public:
  std::vector<QAPSum> train, test;
  std::vector<int> train_ind, test_ind;

  Fold();
  Fold(const std::vector<QAPSum> &train, const std::vector<QAPSum> &test);
  virtual ~Fold();

  /**
   * Run learning on this fold
   */
  std::vector<TestResult> run(Learner &learner) const;

  /**
   * Build a set of folds according the the classic shuffle + split method
   *
   * @arg @c all The full set of QAPs to fold
   * @arg @c n The number of folds to create
   * @arg @c fold_file File into which to write the shuffle order.  May be NULL
   */
  static std::vector<Fold> nfold(const std::vector<QAPSum> &all, int n, std::ostream *fold_file = NULL);

  /**
   * Create a set of folds from a given shuffling.
   */
  static std::vector<Fold> from_shuffle(const std::vector<QAPSum> &all, int n, const std::vector<int> &shuffle);

  /**
   * Load a set of folds from a file containing the shuffle order
   */
  static std::vector<Fold> from_file(const std::vector<QAPSum> &all, int n, std::istream &fold_file);

  /**
   * Run learning across a set of folds, with some informative output
   */
  static std::vector<std::vector<TestResult> > run(const std::vector<Fold> &F, Learner &learner, int nthreads = 1);

  /**
   * Merge the results across multiple runs.
   */
  static std::vector<TestResult> mergeResults(const std::vector<std::vector<TestResult> > &fold_results);

protected:
};

#endif // #ifndef __FOLD_H__
