#include "Learners.hpp"
#include "QAP/StochasticGradient.hpp"

using std::vector;

///////////////////////////// BnBLearner ///////////////////////

BnBLearner::BnBLearner(bool init_greedy, int init_max_visit) :
  greedy(init_greedy), max_visit(init_max_visit), w(NULL)
{
}

BnBLearner::~BnBLearner()
{
  delete w;
}

void BnBLearner::reset()
{
  delete w;
  w = NULL;
}

void BnBLearner::train(const vector<QAPSum> &train, const vector<Permutation> &truth)
{
  if(w == NULL) {
    w = new QAPSum::Weights(train[0].m);
  }

  double lambda = 1e-2;
  int niters = 10000;
  stochasticGradient(train, truth, *w, niters, lambda);
}

Permutation BnBLearner::predict(const QAPSum &qapsum, int *nvisited)
{
  if(w == NULL) {
    w = new QAPSum::Weights(qapsum.m);
  }

  //cout<<"Testing with "<<(*w)<<endl;
  return qapsum.solve(*w, nvisited, greedy, max_visit);
}

Learner *BnBLearner::clone() const
{
  BnBLearner *l = new BnBLearner(greedy, max_visit);
  if(w != NULL) {
    l->w = new QAPSum::Weights(*w);
  }
  return l;
}

////////////////////////////// NoLearner //////////////////////////

NoLearner::NoLearner(bool init_greedy, int init_max_visit) :
  greedy(init_greedy), max_visit(init_max_visit)
{
}

NoLearner::~NoLearner()
{
}

void NoLearner::reset()
{
}

void NoLearner::train(const vector<QAPSum> &qapsum, const vector<Permutation> &truth)
{
}

Permutation NoLearner::predict(const QAPSum &qapsum, int *nvisited)
{
  return qapsum.solve(QAPSum::Weights(qapsum.m), nvisited, greedy, max_visit);
}

Learner *NoLearner::clone() const
{
  return new NoLearner(greedy, max_visit);
}
