#ifndef __LEARNERS_H__
#define __LEARNERS_H__

#include "Fold.hpp"

/**
 * Normal learner based on Deepti's method
 */
class BnBLearner: public Learner
{
public:
  BnBLearner(bool greedy = false, int max_visit = -1);
  virtual ~BnBLearner();

  virtual void reset();

  virtual void train(const std::vector<QAPSum> &qapsum, const std::vector<Permutation> &truth);
  virtual Permutation predict(const QAPSum &qapsum, int *nvisited);

  virtual Learner *clone() const;

private:
  bool greedy;
  int max_visit;
  QAPSum::Weights *w;

  BnBLearner(const BnBLearner &);
  const BnBLearner &operator = (const BnBLearner &);
};

/**
 * Run Kondor's method to solve the QAP with no learning
 */
class NoLearner: public Learner
{
public:
  NoLearner(bool greedy = false, int max_visit = -1);
  virtual ~NoLearner();

  virtual void reset();

  virtual void train(const std::vector<QAPSum> &qapsum, const std::vector<Permutation> &truth);
  virtual Permutation predict(const QAPSum &qapsum, int *nvisited);

  virtual Learner *clone() const;

private:
  bool greedy;
  int max_visit;
};

#endif // #ifndef __LEARNERS_H__
