#include "GilmoreLawler.hpp"
#include "AssignmentMatrix.hpp"
#include "ScalarCounter.hpp"

#include <algorithm>

#ifdef _branchCounter
extern ScalarCounter branchCounter;
#endif

bool maxGLPaircomparison(pair<int,double> a, pair<int,double> b){ return(a.second>b.second); }



FIELD GilmoreLawlerBound(const Fmatrix& G1, const Fmatrix& G2, const Fmatrix& L){
  const int n=G1.Nrows;
  AssignmentMatrix M(n,n);
  for(int i=0; i<n; i++){
    vector<double> a1(n-1); for(int k=0; k<n-1; k++) a1[k]=G1(i,k+(k>=i));
    sort(a1.begin(), a1.end());
    for(int j=0; j<n; j++){ // if pi(i)=j ...
      vector<double> a2(n-1); for(int k=0; k<n-1; k++) a2[k]=G2(j,k+(k>=j));
      sort(a2.begin(), a2.end());
      double t=0; for(int k=0; k<n-1; k++) t+=a1[k]*a2[k];
      M(i,j)=L(i,j)+t;
    }
  }
  //cout<<M.str()<<endl;
  return M.KuhnMunkres();
}



FIELD GilmoreLawlerMaximize(const Fmatrix& G1, const Fmatrix& G2, const Fmatrix& L, const FIELD costsofar, 
			    vector<int>& result, FIELD maxsofar, int* histo, const bool firstshot){
  #ifdef _branchCounter
    branchCounter+=1;
  #endif 

  const int n=G1.Nrows;
  if(histo!=NULL) histo[n]++;

  if(n==1){
    double val=costsofar+G1(0,0)*G2(0,0)+L(0,0);
    if(val>maxsofar){
      maxsofar=val; 
      result.clear();
      result.push_back(1);
    }
    return maxsofar;
  }

  vector<Fmatrix*> G1sub;
  vector<Fmatrix*> G2sub;
  vector<Fmatrix*> Lsub;
  vector<pair<int, double> > bounds;

  for(int jj=1; jj<=n; jj++){
    Fmatrix* G1s=new Fmatrix(n-1,n-1);
    Fmatrix* G2s=new Fmatrix(n-1,n-1);
    Fmatrix* Ls=new Fmatrix(n-1,n-1);
    for(int i=0; i<n-1; i++)
      for(int j=0; j<n-1; j++){
	(*G1s)(i,j)=G1(i+(i>=jj-1),j+(j>=jj-1));
	(*G2s)(i,j)=G2(i,j);
	(*Ls)(i,j)=L(i+(i>=jj-1),j)+G1(i+(i>=jj-1),jj-1)*G2(j,n-1)+G1(jj-1,i+(i>=jj-1))*G2(n-1,j);
      }
    G1sub.push_back(G1s);
    G2sub.push_back(G2s);
    Lsub.push_back(Ls);
    bounds.push_back(pair<int,double>(jj,GilmoreLawlerBound(*G1s,*G2s,*Ls)+costsofar+L(jj-1,n-1)));
  }

  sort(bounds.begin(), bounds.end(), maxGLPaircomparison);
  
  for(int i=0; i<n && bounds[i].second>maxsofar; i++){
    int r=bounds[i].first-1;
    FIELD proposed=GilmoreLawlerMaximize(*G1sub[r],*G2sub[r],*Lsub[r],costsofar+L(r,n-1),result,maxsofar,histo,firstshot);
    if(proposed>maxsofar){
      maxsofar=proposed;
      result.push_back(r+1);
    }
    if(firstshot) break;
  }
  
  for(int i=0; i<n; i++){delete G1sub[i]; delete G2sub[i]; delete Lsub[i];}

  return maxsofar;
}

