#ifndef __QAPSum_HPP__
#define __QAPSum_HPP__

#include <vector>
#include <cassert>
#include <ostream>

#include "Graph.hpp"
#include "QAP.hpp"
#include "Permutation.hpp"

using namespace SnSF;

class QAPBnB;
class QAPbranch;

class QAPSum 
{
public:
  class Weights
  {
  public:
    Weights(int m);
    Weights(int m, const double *w);
    Weights(const Weights &);
    ~Weights();

    const Weights &operator = (const Weights &);

    int length() const;
    double *weights();
    const double *weights() const;
    double norm() const;

    double &operator [] (int i)
    {
      assert(i >= 0 && i < m);
      return w[i];
    }

    double operator [] (int i) const
    {
      assert(i >= 0 && i < m);
      return w[i];
    }

    const Weights &operator += (const Weights &);
    const Weights &operator -= (const Weights &);
    const Weights &operator *= (double);
    const Weights &operator /= (double);

    Weights operator + (const Weights &b)
    {
      Weights sum(*this);
      sum += b;
      return sum;
    }

    Weights operator - (const Weights &subtrahend)
    {
      Weights difference(*this);
      difference -= subtrahend;
      return difference;
    }

  private:
    int m;
    double *w;
  };

  int n, m;
  //vector of QAPs;
  std::vector<QAP> Q;

  /**
   * Construct an empty QAPSum
   */
  QAPSum();

  /**
   * Construct a QAPSum for a QAP vector
   */
  QAPSum(const std::vector<QAP> &Q);

  ~QAPSum();


  /**
   * Find the optimal permutation for the QAP with the given parameters.
   *
   * @see cost
   */
  Permutation solve(const Weights &w, int *nvisited = NULL, bool greedy = true, int max_visit = -1) const;

  /**
   * A step of our learning procedure.
   *
   * Adjusts parameters to get the QAP branch-and-bound to match a true
   * permutation.
   */
  //void train(const Permutation &truth, Weights &w) const;

  /**
   * Return the objective function in the real domain.
   *
   * @todo Parameters?
   */
  SnSF::Function objectiveFunction(const Weights &params);

  /**
   * Permute the problem, such that the true correspondence is permuted by @c P
   */
  void permuteProblem(const Permutation &P);

  // Accessors
  QAP &operator [] (int i)
  {
    assert(i >= 0 && i < Q.size());
    return Q[i];
  }

  const QAP &operator [] (int i) const
  {
    assert(i >= 0 && i < Q.size());
    return Q[i];
  }
};

std::ostream &operator << (std::ostream &, const QAPSum::Weights &w);

#endif // #ifndef __QAP_HPP__
