#include "SquaredGraphCorrelation.hpp"

#include "Math.hpp"


Fourier* SquaredGraphCorrelation(const Transform::Left& Tg2, const Transform::Left& Tg3, const Transform::Left& Tg4, 
				 const Transform::Left& Tc2, const Transform::Left& Tc3, const Transform::Left& Tc4,
				 const Graph& G1, const Graph& G2)
{

  const int n=G1.Nrows;
  const Graph* G[2]; G[0]=&G1; G[1]=&G2;
  Fourier* result;

  {
    Fourier* Ftemp[2];
    for(int g=0; g<2; g++){
      Function f(Tg4);
      for(int i=1; i<=n; i++){
	int ioffset=(n-i)*(n-1)*(n-2)*(n-3);
	for(int j=1; j<=n; j++){
	  if(j==i) continue;
	  int joffset=(n-1-(j-(i<j)))*(n-2)*(n-3)+ioffset;
	  for(int k=1; k<=n; k++){
	    if((k==i)||(k==j)) continue;
	    int koffset=(n-2-(k-(i<k)-(j<k)))*(n-3)+joffset;
	    for(int l=1; l<=n; l++){
	      if((l==i)||(l==j)||(l==k)) continue;
	      f.f[koffset+(n-3-(l-(i<l)-(j<l)-(k<l)))]=G[g]->at(i-1,j-1)*G[g]->at(k-1,l-1);
	    }
	  }
	}
      }
      Ftemp[g]=new Fourier(f);
    }
    result=new Fourier(Tc4.top(),Product<Fourier,Transpose<Fourier> >(*Ftemp[0],~(*Ftemp[1])));
    (*result)*=Factorial()(n-4);
    delete Ftemp[0]; delete Ftemp[1]; 
  }

  for(int c=0; c<4; c++){
    Fourier* Ftemp[2];
    for(int g=0; g<2; g++){
      Function f(Tg3);
      for(int i=1; i<=n; i++){
	int ioffset=(n-i)*(n-1)*(n-2);
	for(int j=1; j<=n; j++){
	  if(j==i) continue;
	  int joffset=(n-1-(j-(i<j)))*(n-2)+ioffset;
	  for(int k=1; k<=n; k++){
	    if((k==i)||(k==j)) continue;
	    switch(c){
	    case 0: f.f[joffset+(n-2-(k-(i<k)-(j<k)))]=G[g]->at(i-1,j-1)*G[g]->at(i-1,k-1); break;
	    case 1: f.f[joffset+(n-2-(k-(i<k)-(j<k)))]=G[g]->at(i-1,j-1)*G[g]->at(j-1,k-1); break;
	    case 2: f.f[joffset+(n-2-(k-(i<k)-(j<k)))]=G[g]->at(i-1,j-1)*G[g]->at(k-1,i-1); break;
	    case 3: f.f[joffset+(n-2-(k-(i<k)-(j<k)))]=G[g]->at(i-1,j-1)*G[g]->at(k-1,j-1); break;
	    }
	  }
	}
      }
      Ftemp[g]=new Fourier(f);
    }
    Fourier Fthird(Tc3.top(),Product<Fourier,Transpose<Fourier> >(*Ftemp[0],~(*Ftemp[1])));
    Fthird*=Factorial()(n-3);
    delete Ftemp[0]; delete Ftemp[1];
    for(int i=0; i<Fthird.size(); i++){
      //cout<<i<<endl; //" "<<(*Fsquared)[i]->T.shape.str()<<" "<<Fthird[i]->T.shape.str()<<" "<<(*Fsquared)[i]->dyadic<<endl;
      *(*result)[i]+=*Fthird[i];}
  }

  for(int c=0; c<2; c++){
    Fourier* Ftemp[2];
    for(int g=0; g<2; g++){
      Function f(Tg2);
      for(int i=1; i<=n; i++) 
	for(int j=1; j<=n; j++){
	  if(j==i) continue;
	  switch(c){
	  case 0: f.f[(n-i)*(n-1)+(n-1-(j-(i<j)))]=G[g]->at(i-1,j-1)*G[g]->at(j-1,i-1); break;
	  case 1: f.f[(n-i)*(n-1)+(n-1-(j-(i<j)))]=G[g]->at(i-1,j-1)*G[g]->at(i-1,j-1); break;
	  }
	}
      Ftemp[g]=new Fourier(f);
    }
    Fourier Fsecond(Tc2.top(),Product<Fourier,Transpose<Fourier> >(*Ftemp[0],~(*Ftemp[1])));
    Fsecond*=Factorial()(n-2);
    delete Ftemp[0]; delete Ftemp[1];
    for(int i=0; i<Fsecond.size(); i++){
      //cout<<i<<" "<<(*Fsquared)[i]->T.shape.str()<<" "<<Fsecond[i]->T.shape.str()<<" "<<(*Fsquared)[i]->dyadic<<endl;
      *(*result)[i]+=*Fsecond[i];}
  }      

  return result;
}
