#include "SnSFclasses.hpp"
#include "Graph.hpp"
#include "Math.hpp"
#include "maxFourier.hpp"
#include "SquaredGraphCorrelation.hpp"
#include "GilmoreLawler.hpp"

#include <cmath>

#define GLqap
#define F1qap

const bool firstshot=0;
const bool KM=1;

using namespace SnSF;

#define _branchCounter
#ifdef _branchCounter
ScalarCounter branchCounter;
#endif

main(int argc, char** argv){

  int n=6; if(argc>=2) sscanf(argv[1],"%d",&n);
  int niter=1; if(argc>=3) sscanf(argv[2],"%d",&niter);
  int subgraphsize=0; if(argc>=4) sscanf(argv[3],"%d",&subgraphsize);

  Transform::Left T(n,n-2);
  Transform::Left Tcorr(OuterProduct<Transform>(T,T));
  double nFact=Factorial()(n);

  double xGL=0;
  double xsqGL=0;
  double xF1=0;
  double xsqF1=0;
  double GLres=0;
  double xdiff=0;
  double xsqdiff=0;
  int bad=0;

  for(int iter=0; iter<niter; iter++){

    Graph::RandomWeighted G1(n);
    for(int i=0; i<n; i++)
      for(int j=0; j<i; j++){
	G1(i,j)=-G1(j,i);
      }
    vector<int> res;
    int histo[n+1];
    double max=-1;
    Fmatrix L(n,n,0);

    Graph::RandomWeighted G2(n);
    //cout<<G2(1,2)<<endl;
    for(int i=0; i<n; i++) 
      for(int j=0; j<n; j++) {
	G2(i,j)=i-j;
	//G2(j,i)=-(G2(i,j)-G2(j,i))/2;
	//G2(i,j)=-G2(j,i);
      }
    //    Graph::Chain G2(n);
    //for(int i=0; i<n; i++)
    //for(int j=0; j<n; j++)
    //if (i!=j) G2(i,j)=1-G2(i,j);
  
#ifdef GLqap
    double GLbound=GilmoreLawlerBound(G1,G2,L);
    xGL+=GLbound;
    xsqGL+=GLbound*GLbound;
    cout<<"GL: "<<GLbound<<" ";
#endif

#ifdef F1qap
    Fourier* F1=G1.Fourier(T);
    Fourier* F2=G2.Fourier(T);
    Fourier F(Tcorr.top(),Product<Fourier,Transpose<Fourier> >(*F1,~*F2));
    F*=Factorial()(n-2);
    double F1bound=FourierUpperBound(F,KM,1);
    xF1+=F1bound;
    xsqF1+=F1bound*F1bound;
    cout<<"F1: "<<F1bound<<" ";
#endif 

    if(GLbound<=F1bound) bad++;
    xdiff+=GLbound-F1bound;
    xsqdiff+=(GLbound-F1bound)*(GLbound-F1bound);
    
    cout<<endl;
  }

  double muGL=xGL/niter;
  double sigGL=sqrt((xsqGL-2*muGL*xGL+muGL*muGL*niter)/niter);
  cout<<"GL: "<<muGL<<" +- "<<sigGL<<endl;
  double muF1=xF1/niter;
  double sigF1=sqrt((xsqF1-2*muF1*xF1+muF1*muF1*niter)/niter);
  cout<<"F1: "<<muF1<<" +- "<<sigF1<<endl;
  double mudiff=xdiff/niter;
  double sigdiff=sqrt((xsqdiff-2*mudiff*xdiff+mudiff*mudiff*niter)/niter);
  cout<<"Del:"<<mudiff<<" +- "<<sigdiff<<endl;
  cout<<"bad:"<<(double)bad/niter<<endl;

  ofstream ofs("log3.txt",std::ios::app);
  ofs<<n<<" "<<niter<<" "<<mudiff<<" "<<sigdiff<<" "<<(double)bad/niter<<endl;
  ofs.close();

}
