#include "SnSFclasses.hpp"
#include "Graph.hpp"
#include "Math.hpp"
#include "maxFourier.hpp"
#include "SquaredGraphCorrelation.hpp"
#include "GilmoreLawler.hpp"

#include <cmath>

#define GLqap
#define F1qap

const bool firstshot=0;
const bool KM=1;

using namespace SnSF;

#define _branchCounter
#ifdef _branchCounter
ScalarCounter branchCounter;
#endif

main(int argc, char** argv){

  int n=6; if(argc>=2) sscanf(argv[1],"%d",&n);
  int niter=1; if(argc>=3) sscanf(argv[2],"%d",&niter);
  int subgraphsize=0; if(argc>=4) sscanf(argv[3],"%d",&subgraphsize);

  Transform::Left T(n,n-2);
  Transform::Left Tcorr(OuterProduct<Transform>(T,T));
  double nFact=Factorial()(n);

  double xGL=0;
  double xsqGL=0;
  double xF1=0;
  double xsqF1=0;
  double GLres=0;
  int bad=0;

  for(int iter=0; iter<niter; iter++){

    Graph::RandomWeighted G1(n);
    for(int i=0; i<n; i++)
      for(int j=0; j<i; j++){
	//G1(i,j)=(G1(i,j)+G1(j,i))/2;
	G1(i,j)=-G1(j,i);
      }

    vector<int> res;
    int histo[n+1];
    double max=-1;
    Fmatrix L(n,n,0);
    
    Graph::RandomWeighted G2(n);
    for(int i=0; i<n; i++) 
      for(int j=0; j<n; j++) {
	G2(i,j)=i-j;
      }


    //    Graph::Chain G2(n);
    //for(int i=0; i<n; i++)
    //  for(int j=0; j<n; j++)
    //if (i!=j) G2(i,j)=1-G2(i,j);

#ifdef GLqap
  res.clear();
  for(int i=1; i<=n; i++) histo[i]=0;
  max=GilmoreLawlerMaximize(G1,G2,L,0,res,0,histo,firstshot);
  cout<<"GL: "<<Permutation(CClist(res))<<" "<<max<<" "<<((double)branchCounter())<<" ";//<<endl;
  //for(int i=1; i<=n; i++) cout<<histo[i]<<"."; cout<<endl;
  GLres=(double)branchCounter();
  xGL+=(double)branchCounter();
  xsqGL+=pow((double)branchCounter(),2);
  branchCounter.reset();    
  cout<<endl;
#endif


#ifdef F1qap
  {
    Fourier* F1=G1.Fourier(T);
    Fourier* F2=G2.Fourier(T);
    Fourier F(Tcorr.top(),Product<Fourier,Transpose<Fourier> >(*F1,~*F2));
    F*=Factorial()(n-2);
    res.clear();
    for(int i=1; i<=n; i++) histo[i]=0;
    max=maxFourier(F,res,0,histo,KM,firstshot);
    cout<<"F1: "<<Permutation(CClist(res))<<" "<<max<<" "<<((double)branchCounter())<<" ";//<<endl;
    //for(int i=1; i<=n; i++) cout<<histo[i]<<"."; cout<<endl;
    if((double)branchCounter()>=GLres) bad++;
    xF1+=(double)branchCounter();
    xsqF1+=pow((double)branchCounter(),2);
    branchCounter.reset();
    cout<<endl;
  }
#endif 


  cout<<endl;
  }

  double muGL=xGL/niter;
  double sigGL=sqrt((xsqGL-2*muGL*xGL+muGL*muGL*niter)/niter);
  cout<<"GL: "<<muGL<<" +- "<<sigGL<<endl;
  double muF1=xF1/niter;
  double sigF1=sqrt((xsqF1-2*muF1*xF1+muF1*muF1*niter)/niter);
  cout<<"F1: "<<muF1<<" +- "<<sigF1<<endl;
  cout<<"bad:"<<((double)bad)/niter<<endl;

  ofstream ofs("log.txt",std::ios::app);
  ofs<<n<<" "<<niter<<" "<<muGL<<" "<<sigGL<<" "<<muF1<<" "<<sigF1<<" "<<bad/niter<<endl;
  ofs.close();

}
