//#include <string>
//#include <fstream>
#include <iostream>
//#include <cstdlib>
//#include <cstdio>
//#include <cmath>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
#include "gsl/gsl_rng.h"
#include "gsl/gsl_randist.h"
#include <gsl/gsl_linalg.h>


using namespace std;

const gsl_rng_type *gslvarT;
gsl_rng *gslvarR;


void print_matrix(gsl_matrix *m, int rows, int cols, const char *stringtoprint)
{
  printf("%s:\n", stringtoprint);
  for (int i=0;i<rows;i++)
    {
      for (int j=0;j<cols;j++)
	{
	  printf("%g\t",gsl_matrix_get(m, i, j));
	}
      printf("\n");
    }
  printf("\n");
}

void copy_matrix(gsl_matrix *src, gsl_matrix *dest, int rows, int cols)
{
  for (int i=0;i<rows;i++)
    {
      for (int j=0;j<cols;j++)
	{
	  gsl_matrix_set(dest, i, j, gsl_matrix_get(src, i, j));
	}
      printf("\n");
    }
  printf("\n");
}

int main()
{
  srand48(time(NULL));
  int rows = 3;
  int cols = 3;

  gsl_matrix *mymatrix = gsl_matrix_alloc(rows,cols);
  gsl_matrix *mymatrixcopy = gsl_matrix_alloc(rows,cols);
  gsl_matrix *inv = gsl_matrix_alloc(rows,cols);
  gsl_permutation *perm = gsl_permutation_alloc (rows);
  int *signum = new int[rows];

  for (int i = 0; i < rows; i++)
    {
      for (int j = 0; j < cols; j++)
	{
	  float val = drand48();
	  //printf("%f\n", val);
	  //getchar();
	  gsl_matrix_set(mymatrix, i, j, val);
	}
    }
  
  copy_matrix(mymatrix, mymatrixcopy, rows, cols);
  print_matrix(mymatrix, rows, cols, "My Input Matrix:");

  gsl_linalg_LU_decomp(mymatrix, perm, signum);
  gsl_linalg_LU_invert(mymatrix, perm, inv);
  print_matrix(mymatrix, rows, cols, "LU of Input Matrix:");

  //if PSD then
  //gsl_linalg_cholesky_decomp() followed by gsl_linalg_cholesky_invert() 

  print_matrix(mymatrixcopy, rows, cols, "My Input Matrix:");
  cout << endl;
  print_matrix(inv, rows, cols, "Matrix Inverse:");

  FILE *fout = fopen("output.txt","wt");
  gsl_matrix_fprintf(fout, mymatrixcopy, "%f");
  fclose(fout);
     
  gsl_permutation_free (perm);
  gsl_matrix_free (mymatrix);
  gsl_matrix_free (inv);

  delete signum;
  return 0;
  
}
