//g++ -I/usr/local/include/eigen3 matrixinvexample3.cpp -o matrixinvexample3 -lm
#ifndef EIGEN_YES_I_KNOW_SPARSE_MODULE_IS_NOT_STABLE_YET
#define EIGEN_YES_I_KNOW_SPARSE_MODULE_IS_NOT_STABLE_YET
#endif

#include <iostream>
#include <Eigen/SVD>
#include <Eigen/Sparse>
#include <Eigen/StdVector>
#include <Eigen/Dense>
#include <Eigen/LU>
#include <Eigen/QR>
#include <Eigen/Cholesky>
#include <Eigen/Eigenvalues>
#include <Eigen/Householder>
#include <Eigen/StdDeque>  
#include <Eigen/StdList>  

#include <Eigen/Core>

using namespace Eigen;
using namespace std;

template <typename CMatrix, typename CINVMatrix>
void pseudo_inverse(const CMatrix &C, CINVMatrix &CINV)
{
  typedef typename CMatrix::Scalar Scalar;
  typedef typename CMatrix::Index Index;
  typedef Matrix<Scalar,Dynamic,1> TmpVec;

  Index rows = C.rows(), cols = C.cols();

  TmpVec d(rows), e(rows), l(cols), p(rows), q(rows), r(rows);
  Scalar rho, rho_1, alpha;
  d.setZero();

  for (Index i = 0; i < rows; ++i)
  {
    d[i] = 1.0;
    rho = 1.0;
    e.setZero();
    r = d;
    p = d;

    while (rho >= 1e-38)
    { /* conjugate gradient to compute e             */
      /* which is the i-th row of inv(C * trans(C))  */
      l = C.transpose() * p;
      q = C * l;
      alpha = rho / p.dot(q);
      e +=  alpha * p;
      r += -alpha * q;
      rho_1 = rho;
      rho = r.dot(r);
      p = (rho/rho_1) * p + r;
    }

    l = C.transpose() * e; // l is the i-th row of CINV

    for (Index j=0; j<l.size(); ++j)
      if (l[j]<1e-15)
        CINV(i,j) = l[j];

    d[i] = 0.0;
  }
}


int main()
{
  int rows = 4;
  int cols = 4;
  MatrixXd m(rows,cols);
  MatrixXd m2(rows,cols);
  
  //m(0,0) = 3;
  //m(1,0) = 2.5;
  //m(0,1) = -1;
  //m(1,1) = m(1,0) + m(0,1);
  m(0,0) = -0.125;
  m(0,1) = -0.1613743060919757;
  m(0,2) = -0.2282177322938192;
  m(0,3) = -0.3952847075210473;

  m(1,0) = -0.8068715304598786;
  m(1,1) = 0.06944444444444442;
  m(1,2) = 0.0982092751647983;
  m(1,3) = 0.170103454359943;

  m(2,0) = -1.141088661469096;
  m(2,1) = 0.09820927516479826;
  m(2,2) = 0.1388888888888889;
  m(2,3) = 0.2405626121623441;

  m(3,0) = -1.976423537605236;
  m(3,1) =  0.1701034543599428;
  m(3,2) = 0.240562612162344;
  m(3,3) = 0.4166666666666665;
  
  cout << "Given Matrix" << endl;
  std::cout << m << std::endl;
  
  JacobiSVD< MatrixXd > svd(m);
  MatrixXd sigma = MatrixXd::Zero(rows,cols);
  MatrixXd matU  = MatrixXd::Zero(rows,rows);
  sigma.diagonal() = svd.singularValues();
  svd.compute(m, Eigen::ComputeThinU | Eigen::ComputeThinV);
  matU = svd.matrixU();

  MatrixXd avalidation(rows,cols);
  avalidation = matU * sigma * svd.matrixV().transpose();
  cout << endl << "Does SVD reconstruct original Matrix?" << endl;
  cout << avalidation << endl;
  cout << "testSingularMatrixSVD() : ";
  if((m).isApprox(avalidation, 1e-12))
    {
      cout << "Success." << endl;
      //return true;
    }
  else
    {
      cout << "Fail." << endl;
      //return false;
    }
  
  cout << "Standard Inverse" << endl;
  std::cout << m.inverse() << endl;
   
  cout << "Singular Values diagonal matrix:" << endl;
  cout << sigma << endl << endl;
  
  MatrixXd sigmainv = MatrixXd::Zero(rows,cols);
  MatrixXd sigmatemp = MatrixXd::Zero(rows,cols);
  
  for (int i = 0; i < rows; i++)
    {
      if (sigma(i,i) > 1e-6)
	{
	  sigmainv(i,i) = 1./sigma(i,i);
	  sigmatemp(i,i) = sigma(i,i);
	}
    }
  
  cout << "Reciprocal of Singular Values diagonal matrix (after rounding out)" << endl;
  cout << sigmainv << endl << endl;
  
  
  MatrixXd invoutmat(rows,cols);
  pseudo_inverse(m,invoutmat);
  
  cout << "From customized Pseudo Inv function" << endl;
  cout << invoutmat << endl;
  
  cout << "Pseudo Inverse with rounding entries less than 1e-6" << endl;
  MatrixXd avalidation2(rows,cols);
  avalidation2 = matU * sigmainv.transpose() * svd.matrixV().transpose();
  cout << avalidation2 << endl;
  
  cout << "Reconstruction using rounded singular diagonal matrix" << endl;
  cout <<  matU * sigmatemp * svd.matrixV().transpose() << endl << endl;
  
  cout << "Original again: " << endl;
  cout <<   m << endl;
  
  return 0;
  
}
