#include "maxFourier.hpp"
#include "AssignmentMatrix.hpp"
#include "Math.hpp"
#include "Permutation.hpp"

#include <cmath>
#include <algorithm>
using std::sort;

#ifdef _branchCounter
extern ScalarCounter branchCounter;
#endif

bool maxFourierPaircomparison(pair<int,double> a, pair<int,double> b){ return(a.second>b.second); }


using namespace SnSF;

FIELD maxFourier(const Fourier& F, vector<int>& result, FIELD maxsofar, int* histo, const bool KM, const bool firstshot){ 


#ifdef _branchCounter
  branchCounter+=1;
#endif 


  const int n=F.T.n;
  //cout<<n<<endl;
  //cout<<str()<<endl;
  //cout<<upperBound()<<endl;

  if(histo!=NULL) histo[n]++;

#if 0
  if(n==3 && F.T.order==6){
    const double C1=sqrt(3)/2;
    const double aa=F.at(0)->at(0,0);
    const double ba=F.at(1)->at(0,0);
    const double bb=F.at(1)->at(0,1);
    const double bc=F.at(1)->at(1,0);
    const double bd=F.at(1)->at(1,1);
    const double ca=F.at(2)->val(0,0);
    const double f0=(aa+2*ba+2*bd+ca)/6;
    const double f1=(aa+2*ba-2*bd-ca)/6;
    const double f2=(aa-ba+2*C1*bc+2*C1*bb+bd-ca)/6;
    const double f3=(aa-ba+2*C1*bc-2*C1*bb-bd+ca)/6;
    const double f4=(aa-ba-2*C1*bc+2*C1*bb-bd-ca)/6;
    const double f5=(aa-ba-2*C1*bc-2*C1*bb+bd+ca)/6;
    if(f0>maxsofar){maxsofar=f0; result.clear(); result.push_back(1); result.push_back(2); result.push_back(3);}
    if(f1>maxsofar){maxsofar=f1; result.clear(); result.push_back(1); result.push_back(1); result.push_back(3);}
    if(f2>maxsofar){maxsofar=f2; result.clear(); result.push_back(1); result.push_back(2); result.push_back(2);}
    if(f3>maxsofar){maxsofar=f3; result.clear(); result.push_back(1); result.push_back(1); result.push_back(2);}
    if(f4>maxsofar){maxsofar=f4; result.clear(); result.push_back(1); result.push_back(2); result.push_back(1);}
    if(f5>maxsofar){maxsofar=f5; result.clear(); result.push_back(1); result.push_back(1); result.push_back(1);}
    return maxsofar;
  }

  if(n==2 && F.T.order==2){
    double f0=(F.at(0)->at(0,0)+F.at(1)->at(0,0))/2;
    double f1=(F.at(0)->at(0,0)-F.at(1)->at(0,0))/2;
    if(f0>maxsofar){maxsofar=f0; result.clear(); result.push_back(1); result.push_back(2);}
    if(f1>maxsofar){maxsofar=f1; result.clear(); result.push_back(1); result.push_back(1);}
    return maxsofar;
  }
#endif

  if(F.T.order==1){
    double norm=F[0]->at(0,0);
    if(norm>maxsofar){
      maxsofar=norm; 
      result.clear();
      result.push_back(1);
    }
    return maxsofar;
  }

  vector<Fourier*> child;
  vector<pair<int, double> > norms;
  for(int j=1; j<=n; j++){
    //cout<<j<<endl;
    Fourier subtilde(F);
    //cout<<"aa"<<endl;
    subtilde.rotateLeft(j,1);
    //cout<<"bb"<<endl;
    Fourier* sub=new Fourier(*F.T.descendant,subtilde);
    //cout<<"cc"<<endl;
    child.push_back(sub);
    double bound=FourierUpperBound(*sub,KM);
    norms.push_back(pair<int, double>(j,bound));
  }
  sort(norms.begin(), norms.end(), maxFourierPaircomparison);
  
  for(int i=0; i<n && norms[i].second>maxsofar; i++){
    FIELD proposed=maxFourier(*child[norms[i].first-1],result,maxsofar,histo,KM,firstshot);
    if(proposed>maxsofar){
      maxsofar=proposed;
      result.push_back(norms[i].first);
    }
    if(firstshot) break;
  }
  
  for(int i=0; i<child.size(); i++) delete child[i];

  return maxsofar;
}



FIELD FourierUpperBound(const Fourier& F, const bool KM, const bool verbose){
  FIELD result=0;
  int n=F.T.n;
  //cout<<"FourierUpperBound KM "<<KM<<endl;
  //getchar();
  if(n>=4 && KM){
    if (verbose) cout<<"( ";
    result=F.at(0)->at(0,0)/((double)F.T.order);
    AssignmentMatrix M(n,n);
    for(int i=1; i<=n; i++){
      for(int j=1; j<=n; j++){
	Component subtilde(*F[1]);
	subtilde.rotateLeft(i,1);
	subtilde.rotateRight(j,1);
	double v=subtilde(0,0);
	M(i-1,j-1)=v/((double)F.T.order); 
      }
    }
    result+=M.KuhnMunkres()*(n-1);
    if (verbose) cout<<result<<" ";
    for(int i=2; i<F.size(); i++){
      double tracenorm=F[i]->T.Nrows*F[i]->tracenorm()/((double)F.T.order);
      result+=tracenorm;
      if (verbose) cout<<tracenorm<<" ";
    }
    if (verbose) cout<<") ";
  }else{
    if (verbose) cout<<"( ";
    //cout<<"FourierUpperBound F.size "<<F.size()<<endl;
    //getchar();
    for(int c=0; c<F.size(); c++){
      //cout<<"FourierUpperBound T.Nrows*F[c] "<<*(F[c]->T.Nrows*F[c])<<endl;
      //getchar();
      double tracenorm=F[c]->T.Nrows*F[c]->tracenorm()/((double)F.T.order);
      //cout<<"FourierUpperBound tracenorm "<<tracenorm<<endl;
      //getchar();
      if (verbose) cout<<tracenorm<<" ";
      result+=tracenorm;
    }
    if (verbose) cout<<") ";
  }
  return result;
}


FIELD FourierUpperBound2(const Fourier& F){
  FIELD result=0;
  for(int c=0; c<F.size(); c++)
    result+=(F[c]->norm2())*pow(F[c]->T.Nrows,1.5);
  return result/((double)F.T.order);
}



FIELD eval(const Fourier& _F, vector<int> res, const bool verbose){
  FIELD result=0;
  Fourier F(_F);
  const int n=F.T.n;
  for(int i=0; i<res.size()-1; i++){
    F.rotateLeft(res[res.size()-i-1],1,n-i);}
  if(verbose) cout<<"( ";
  for(int c=0; c<F.size(); c++){
    FIELD trace=F[c]->T.Nrows*F[c]->trace()/((double)F.T.order);
    result+=trace;
    if(verbose) cout<<trace<<" ";
  }
  if(verbose) cout<<") ";
  return result;
}


FIELD maxByComp(const Fourier& F){
  const int n=F.T.n;
  double nFact=Factorial()(n);
  FIELD result=F[0]->at(0,0)/nFact;
  cout<<"( "<<F[0]->at(0,0)/nFact<<" ";
  for(int c=1; c<F.size(); c++){
    FIELD max=-1;
    for(int p=1; p<=(int)nFact; p++){
      Component comp(*F[c]);
      Permutation pi(pair<int,int>(n,p));
      vector<int> cclist(pi.cclist());
      for(int i=0; i<n; i++)
	comp.rotateLeft(cclist[n-1-i],1,n-i);
      FIELD trace=comp.Nrows*comp.trace()/nFact;
      if(max==-1 || trace>max) max=trace;
    }
    cout<<max<<" ";
    result+=max;
  }
  cout<<") ";
  return result;
}



FIELD globalMax(const Fourier& _F){
  const int n=_F.T.n;
  double nFact=Factorial()(n);
  FIELD max=-1;
  int best=1;
  for(int p=1; p<=(int)nFact; p++){
    Fourier F(_F);
    Permutation pi(pair<int,int>(n,p));
    vector<int> cclist(pi.cclist());
    for(int i=0; i<n; i++)
      F.rotateLeft(cclist[n-1-i],1,n-i);
    FIELD t=0;
    for(int c=0; c<F.size(); c++)
      t+=F[c]->Nrows*F[c]->trace()/nFact;
    //cout<<pi<<t<<endl;
    if(max==-1 || t>max) {max=t; best=p;}
  }
  //cout<<Permutation(pair<int,int>(n,best))<<" ";
  return max;
}


//FIELD verify(const Graph& G1, const Graph& G2, const Permutation& p;){
//  G2.permute(pi);
//}
