#include <string>
#include <fstream>
#include <cmath>
#include <sstream>

#include "ScalarCounter.hpp"
#include "learn.hpp"

ScalarCounter branchCounter;

using std::cout; using std::endl;

const char * const USAGE = "\
Usage: testset filelist permfile\n\
\n\
permfile: Output file for permutations.\n\
";


int main(int argc, char **argv)
{
  if(argc != 3 || string(argv[1]) == "-h" || string(argv[1]) == "--help") {
    cout<<USAGE;
    return 0;
  }

  // Read training set
  std::list<QAP> testset = loadFileList(argv[1]);

  cout<<"# test examples: "<<testset.size()<<endl;

  // Perform prediction
  std::vector<Permutation> predict = munkres(testset);
  assert(predict.size() == testset.size());

  // Construct true permutation
  // TODO: read truth from file
  std::vector<Permutation> truth;
  for(std::list<QAP>::iterator i = testset.begin(); i != testset.end(); i++) {
    truth.push_back(Permutation(i->n));
  }
  double l = loss(truth, predict);
  cout<<"loss = "<<l<<endl;

  // Write permutation
  std::ofstream permfile(argv[2], std::ios::out | std::ios::trunc);
  for(int i = 0; i < predict.size(); i++) {
    permfile<<i<<": ";

    for(int j = 0; j < predict[i].n; j++) {
      permfile<<predict[i][j]<<' ';
    }
    permfile<<endl;
  }

  return 0;
}
