#include "SnSFclasses.hpp"
#include "Graph.hpp"
#include "Math.hpp"
#include "maxFourier.hpp"
#include "SquaredGraphCorrelation.hpp"
#include "GilmoreLawler.hpp"

#include <cmath>

using namespace SnSF;

#define _branchCounter
#ifdef _branchCounter
ScalarCounter branchCounter;
#endif

#define TSPqap
//#define Permuteqap
//#define Randomqap

#define F1qap
//#define F1qap2
//#define F2qap
#define GLqap

const bool firstshot=0;
const bool KM=1;


main(int argc, char** argv){

  int n=6; if(argc>=2) sscanf(argv[1],"%d",&n);
  int niter=1; if(argc>=3) sscanf(argv[2],"%d",&niter);
  int subgraphsize=0; if(argc>=4) sscanf(argv[3],"%d",&subgraphsize);

  Transform::Left T(n,n-2);
  Transform::Left Tcorr(OuterProduct<Transform>(T,T));
  double nFact=Factorial()(n);
#ifdef F2qap
  Transform::Left Tg2(n,n-2);
  Transform::Left Tg3(n,n-3);
  Transform::Left Tg4(n,n-4);
  Transform::Left Tc2(OuterProduct<Transform>(Tg2,Tg2));
  Transform::Left Tc3(OuterProduct<Transform>(Tg3,Tg3));
  Transform::Left Tc4(OuterProduct<Transform>(Tg4,Tg4));
#endif
  cout<<"Transforms constructed"<<endl;
  

  for(int iter=0; iter<niter; iter++){

    Graph::RandomWeighted G1(n);
    for(int i=0; i<n; i++)
      for(int j=0; j<i; j++){
	//G1(i,j)=G1(j,i);
      }
  vector<int> res;
  int histo[n+1];
  double max=-1;
  Fmatrix L(n,n,0);

#ifdef TSPqap
  Graph::Chain G2(n);
  for(int i=0; i<n; i++)
    for(int j=0; j<n; j++)
      if (i!=j) G2(i,j)=1-G2(i,j);
#endif

#ifdef Permuteqap
  Graph::RandomWeighted G2(G1);
  cout<<"Target: "<<pow(G1.norm2(),2)<<endl;
  Permutation::Random pi(n);
  G1 = G1.permute(pi); // inverse problem should be G2!!!
  cout<<"pi= "<<pi<<endl;
#endif

#ifdef Randomqap
  Graph::RandomWeighted G2(n);
  //for(int i=0; i<n; i++)
  //  for(int j=0; j<i; j++)
  //G2(i,j)=G2(j,i);
  if(subgraphsize>0)
    for(int i=0; i<n; i++)
      for(int j=0; j<n; j++)
	if(i>=subgraphsize || j>=subgraphsize) G2(i,j)=0;
#endif



#ifdef GLqap
  res.clear();
  for(int i=1; i<=n; i++) histo[i]=0;
  max=GilmoreLawlerMaximize(G1,G2,L,0,res,0,histo,firstshot);
  cout<<"GL: "<<Permutation(CClist(res))<<" "<<max<<" "<<((double)branchCounter())<<" ";//<<endl;
  for(int i=1; i<=n; i++) cout<<histo[i]<<"."; cout<<endl;
  branchCounter.reset();    
#endif


#ifdef F1qap
  {
    Fourier* F1=G1.Fourier(T);
    Fourier* F2=G2.Fourier(T);
    Fourier F(Tcorr.top(),Product<Fourier,Transpose<Fourier> >(*F1,~*F2));
    F*=Factorial()(n-2);
    res.clear();
    for(int i=1; i<=n; i++) histo[i]=0;
    max=maxFourier(F,res,0,histo,KM,firstshot);
    cout<<"F1: "<<Permutation(CClist(res))<<" "<<max<<" "<<((double)branchCounter())<<" ";//<<endl;
    for(int i=1; i<=n; i++) cout<<histo[i]<<"."; cout<<endl;
    branchCounter.reset();
  }
#endif 


#ifdef F1qap2
  Fourier* F1=G1.Fourier(T);
  Fourier* F2=G2.Fourier(T);
  Fourier F(Tcorr.top(),Product<Fourier,Transpose<Fourier> >(*F1,~*F2));
  F*=Factorial()(n-2);
  res.clear();
  for(int i=1; i<=n; i++) histo[i]=0;
  max=maxFourier(F,res,0,histo,KM,0);
  cout<<"F1: "<<Permutation(CClist(res))<<" "<<max<<" "<<((double)branchCounter())<<" ";//<<endl;
  for(int i=1; i<=n; i++) cout<<histo[i]<<"."; cout<<endl;
  branchCounter.reset();
  //cout<<"GL bound: "<<GilmoreLawlerBound(G1,G2,L)<<endl;
  //cout<<"F1 bound: "; cout.flush(); cout<<FourierUpperBound(F,0,1)<<endl;
  //cout<<"By comp:  "; cout.flush(); cout<<maxByComp(F)<<endl;
  //cout<<"Truth:    "; cout.flush(); cout<<eval(F,res,1)<<endl;
  //cout<<"Global mx:"; cout.flush(); cout<<globalMax(F)<<endl;
  //G2.permute(Permutation(CClist(res)));
  //cout<<"Verify:   "; cout.flush(); cout<<G1.inp(G2)<<endl;
  //cout<<Function(F)<<endl;
#endif 


#ifdef F2qap
  Fourier* Fsq=SquaredGraphCorrelation(Tg2,Tg3,Tg4,Tc2,Tc3,Tc4,G1,G2);
  res.clear();
  for(int i=1; i<=n; i++) histo[i]=0;
  max=maxFourier(*Fsq,res,0,histo,KM);
  cout<<"F2: "<<Permutation(CClist(res))<<" "<<max<<" "<<((double)branchCounter())<<" "<<endl;
  for(int i=1; i<=n; i++) cout<<histo[i]<<"."; cout<<endl;
  branchCounter.reset();    
#endif
  
  cout<<endl<<endl;
  }

}



