#include "QAP.hpp"
#include "Timer.hpp"

#include <cstdlib>
#include <ctime>
#include <cassert>

#include <algorithm>


const int n = 8;
const int nedges = 2*n;

ScalarCounter branchCounter;

int main(int argc, char **argv)
{
  unsigned int seed = (unsigned int)time(NULL);
  //seed = 1306855137;
  cout<<"seed = "<<seed<<endl;
  srand(seed);

  Graph::RandomNedges G1(n, nedges);
  Permutation::Random truth(n);
  Graph G2 = G1.permute(truth);

  cout<<G1<<endl;
  cout<<G2<<endl;

  Graph Phi(n, 0);
  Phi(1,2) = 1;

  // Solve the QAP
  QAP qap(G1, G2);
  //QAP qap(G1, G2, Phi);
  WTree w(qap);

  cout<<"Solving QAP..."<<endl;
  Timer timer;
  Permutation bnb_sol = qap.solve(&w);
  cout<<Timer::timeString(timer.elapsed())<<endl;

  // Find objective function
  Function obj = qap.objectiveFunction(&w);
  cout<<"min = "<<obj.minimum()<<endl;
  cout<<"max = "<<obj.maximum()<<endl;

  cout<<"truth: "<<truth<<endl;
  cout<<"truth objective: "<<obj.eval(truth)<<endl;

  cout<<"BnB: "<<bnb_sol<<endl;
  cout<<"BnB solution objective = "<<obj.eval(bnb_sol)<<endl;

  Permutation enum_sol = obj.maximumPermutation();
  cout<<"enum solution = "<<enum_sol<<endl;
  cout<<"enum solution objective = "<<obj.eval(enum_sol)<<endl;

  return 0;
}
