#include <string>
#include <fstream>
#include <cmath>
#include <sstream>

#include "ScalarCounter.hpp"
#include "learn.hpp"

ScalarCounter branchCounter;

using std::cout; using std::endl;

const char * const USAGE = "\
Usage: trainset filelist oldw neww nepochs\n\
\n\
oldw: Pass NULL to start with default w.\n\
";

int main(int argc, char **argv)
{
  if(argc != 5 || string(argv[1]) == "-h" || string(argv[1]) == "--help") {
    cout<<USAGE;
    return 0;
  }

  // Load QAP
  cout<<"Reading training set from "<<argv[1]<<endl;
  cout<<"oldw    "<<argv[2]<<endl;
  cout<<"neww    "<<argv[3]<<endl;
  cout<<"nepochs "<<argv[4]<<endl;

  // Read nepochs
  int nepochs;
  {
    string nepochs_str = argv[4];
    std::stringstream nepochs_stream(nepochs_str);
    nepochs_stream>>nepochs;
  }

  // Read training set
  std::list<QAP> trainset = loadFileList(argv[1]);

  cout<<"# nodes: "<<trainset.front().n<<endl;
  cout<<"Size of training set: "<<trainset.size()<<endl;

  // Construct true permutation
  // TODO: read truth from file
  std::vector<Permutation> truth;
  for(std::list<QAP>::iterator i = trainset.begin(); i != trainset.end(); i++) {
    truth.push_back(Permutation(i->n));
  }

  // Load w
  WTree w(trainset.front());
  if(string(argv[2]) != "NULL") {
    std::ifstream win(argv[2], std::ios::in);
    w.read(win);
  }

  // Perform training
  train(trainset, truth, nepochs, &w);

  std::ofstream wout(argv[3], std::ios::out | std::ios::trunc);
  w.write(wout);

  return 0;
}
