#include <cassert>

#include "Result.hpp"

int TestResult::nwrong() const
{
  assert(predict.n == truth.n);

  int nwrong = 0;
  for(int i = 0; i < predict.n; i++) {
    if(predict[i] != truth[i]) {
      nwrong++;
    }
  }
  return nwrong;
}

double TestResult::accuracy() const
{
  return 1 - nwrong()/((double)predict.n);
}

double TestResult::accuracy(std::vector<TestResult>::const_iterator start,
			    std::vector<TestResult>::const_iterator end)
{
  int nwrong = 0, total = 0;

  std::vector<TestResult>::const_iterator i;
  for(i = start; i != end; i++) {
    nwrong += i->nwrong();
    total += i->predict.n;
  }
  
  return 1 - (double)nwrong/total;
}

double TestResult::averageVisited(std::vector<TestResult>::const_iterator start,
				  std::vector<TestResult>::const_iterator end)
{
  int nvisited = 0, total = 0;

  std::vector<TestResult>::const_iterator i;
  for(i = start; i != end; i++) {
    nvisited += i->nvisited;
    total++;
  }

  return (double)nvisited/total;
}

std::ostream &operator << (std::ostream &stream, const TestResult &r)
{
  stream<<r.accuracy();
  stream<<' '<<r.nvisited;
  stream<<' '<<r.predict;

  return stream;
}
