#ifndef __RESULT_HPP__
#define __RESULT_HPP__

#include <vector>
#include <ostream>

#include "utility/Permutation.hpp"

class TestResult
{
public:
  Permutation truth;
  Permutation predict;

  /**
   * The number of nodes visited to perform prediction
   */
  int nvisited;

  /**
   * The number of wrong entries in the predicted permutation.
   */
  int nwrong() const;

  /**
   * Get the accuracy of the predicted permutation.
   */
  double accuracy() const;

  /**
   * Get the total accuracy across a test set
   */
  static double accuracy(std::vector<TestResult>::const_iterator start,
			 std::vector<TestResult>::const_iterator end);

  /**
   * Get the average number of nodes visited across a test set
   */
  static double averageVisited(std::vector<TestResult>::const_iterator start,
			       std::vector<TestResult>::const_iterator end);
};

std::ostream &operator << (std::ostream &stream, const TestResult &r);

#endif // #ifndef __RESULT_HPP__
