#include "SnSFcomponent.hpp"
#include "SnSFfourier.hpp"
#include "SnSFfourierT.hpp"

#ifdef _ops
extern ScalarCounter ops;
#endif


SnSF::Component::Component(const ComponentT &_T) :
  T(_T), FGmatrix(_T.dyadic, _T.Nrows, _T.Ncols)
{}

SnSF::Component::Component(const ComponentT &_T, const FIELD val) :
  T(_T), FGmatrix(_T.dyadic, _T.Nrows, _T.Ncols, val)
{}


SnSF::Component::Component(const ComponentT& _T, const Identity& dummy): 
  T(_T),FGmatrix(_T.dyadic, _T.Nrows, _T.Ncols, 0){
  if(dyadic) cout<<"Dyadic identity matrices not supported"<<endl;

  for(int i=0; i<Nrows; i++)
    for(int j=0; j<Ncols; j++)
      if (T.rowlabels[i]==T.columnlabels[j]){
	a[i*Ncols+j]=1;
	break;
      }
}



SnSF::Component::Component(const ComponentT& _T, const Fourier& base): 
  T(_T), FGmatrix(_T.dyadic,_T.Nrows,_T.Ncols){
  if(dyadic){cout<<"Error: dyadic matrices not supported in up transforms"<<endl; exit(-1);}
  
  //getchar();
  int offset=0;
  FIELD* current=a;
  for(int subi=0; subi<T.sub.size(); subi++){
    ComponentT& subT=*(base.T[T.sub[subi]]);
    int nrows=subT.Nrows;
    int ncols=subT.Ncols;
    if(!subT.pseudo>0){
      Component& sub=*base[T.sub[subi]];
      for(int i=0; i<nrows; i++){
	for(int j=0; j<offset; j++) current[j]=0; current+=offset;
	for(int j=0; j<ncols; j++) current[j]=sub(i,j); current+=ncols;
	for(int j=0; j<Ncols-offset-ncols; j++) current[j]=0; current+=Ncols-offset-ncols; 
      }
    }else{
      for(int i=0; i<nrows*Ncols; i++) current[i]=0;
      current+=nrows*Ncols;
    }
    offset+=ncols;
  }
}

SnSF::Component::Component(const ComponentT& _T, const Sum<Component, Component>& args):
    T(_T),FGmatrix(Sum<FGmatrix,FGmatrix>(args.left,args.right)){}

SnSF::Component::Component(const ComponentT& _T, const Product<Component, Component>& args):
    T(_T),FGmatrix(Product<FGmatrix,FGmatrix>(args.left,args.right)){}

SnSF::Component::Component(const ComponentT& _T, const Product<Component, Transpose<Component> >& args):
    T(_T),FGmatrix(Product<FGmatrix,Transpose<FGmatrix> >(args.left,Transpose<FGmatrix>(args.right.val))){}

SnSF::Component::Component(const ComponentT& _T, const Product<Component, Inverse<Component> >& args):
  T(_T),FGmatrix(Product<FGmatrix,Inverse<FGmatrix> >(args.left,Inverse<FGmatrix>(args.right.val))){}

SnSF::Component::Component(
  const ComponentT &_T,
  const Product<Inverse<Component>, Component> &args) :

  T(_T),
  FGmatrix(Product<Inverse<FGmatrix>, FGmatrix>(
    Inverse<FGmatrix>(args.left.val),
    args.right
    )) 
{}

SnSF::Component::Component(const ComponentT& _T, const Product<Transpose<Component>, Component>& args):
    T(_T),FGmatrix(Product<Transpose<FGmatrix>,FGmatrix>(Transpose<FGmatrix>(args.left.val),args.right)){}

SnSF::Component::Component(const ComponentT& _T, const Product<Transpose<Component>, Transpose<Component> >& args):
    T(_T),FGmatrix(Product<Transpose<FGmatrix>,Transpose<FGmatrix> >(Transpose<FGmatrix>(args.left.val),Transpose<FGmatrix>(args.right.val))){}

SnSF::Component::Component(const ComponentT& _T, const Product<Component, FIELD>& args):
    T(_T),FGmatrix(Product<FGmatrix,FIELD>(args.left,args.right)){}


SnSF::Component::Component(const ComponentT &_T, const HadamardProduct<Component, Component> &args) :
  FGmatrix(HadamardProduct<FGmatrix, FGmatrix>(args.left, args.right)),
  T(_T)
{
}

SnSF::Component::Component(const ComponentT &_T, const HadamardDivision<Component, Component> &args) :
  T(_T), FGmatrix(HadamardDivision<FGmatrix, FGmatrix>(args.left, args.right)){}

SnSF::Component& SnSF::Component::rotateLeft(const int j, const bool inverse, int m){
  if(m==0) m=T.n;
  //  cout<<"Componenetinverse "<<inverse<<endl;
//   cout<<"SnSF componenet -m, j,inverse "<<m<<" "<<j<<" "<<inverse<<endl;
//   getchar();
  bool done[Nrows];
  for(int p=m-1; p>=j; p--){
    int tau=p;
    if (inverse) tau=j+m-1-p;
    for(int Tab=0; Tab<Nrows; Tab++) done[Tab]=0; 
    for(int Tab=0; Tab<Nrows; Tab++){
      if(!done[Tab]){
	double c1,c2; 
	const int Tdash=T.YORleft(tau,Tab,c1,c2);
// 	cout<<"c1 "<<c1<<endl;
	
	if(Tdash==-1){
	  if(dyadic){
	    a[Tab]=c1*a[Tab];
            #ifdef _ops
	      ops+=1;
            #endif
	  }else{
	    for(int i=Tab*Ncols; i<Tab*Ncols+Ncols; i++) a[i]=c1*a[i];
            #ifdef _ops
	      ops+=Ncols;
            #endif
	  }
	}else{
	  if(dyadic){
	    FIELD temp=a[Tab];
	    a[Tab]=c1*temp+c2*a[Tdash];
	    a[Tdash]=-c1*a[Tdash]+c2*temp;
            #ifdef _ops
	      ops+=2;
            #endif
	  }else{
	    for(int i=0; i<Ncols; i++){
	      FIELD temp=at(Tab,i);
	      at(Tab,i)=c1*temp+c2*at(Tdash,i);
	      at(Tdash,i)=-c1*at(Tdash,i)+c2*temp;
	    }
            #ifdef _ops
	      ops+=2*Ncols;
            #endif
	  }
	  done[Tdash]=1;
	}
      }
    }
  }
 //  cout<<"this "<<*this<<endl;
  return *this;
}



SnSF::Component& SnSF::Component::rotateRight(const int j, const bool inverse, int m){
  if(m==0) m=T.n;
  bool done[Ncols];
  for(int p=m-1; p>=j; p--){
    int tau=p;
    if (inverse) tau=j+m-1-p;
    //cout<<"tau="<<tau<<endl;
    for(int Tab=0; Tab<Ncols; Tab++) done[Tab]=0; 
    for(int Tab=0; Tab<Ncols; Tab++){
      if(!done[Tab]){
	double c1,c2; 
	const int Tdash=T.YORright(tau,Tab,c1,c2);
	if(Tdash==-1){
	  if(dyadic){
	cout<<"Warning: dyadic right rotations not supported yet!"<<endl;
	  }else{
	    for(int i=0; i<Nrows; i++) a[i*Ncols+Tab]=c1*a[i*Ncols+Tab]; // changed 1/7/10 
            #ifdef _ops
	      ops+=Ncols;
            #endif
	  }
	}else{
	  if(dyadic){
	cout<<"Warning: dyadic right rotations not supported yet!"<<endl;
	  }else{
	    for(int i=0; i<Nrows; i++){
	      FIELD temp=at(i,Tab);
	      at(i,Tab)=c1*temp+c2*at(i,Tdash);
	      at(i,Tdash)=-c1*at(i,Tdash)+c2*temp;
	    }
            #ifdef _ops
	      ops+=2*Ncols;
            #endif
	  }
	  done[Tdash]=1;
	}
      }
    }
  }
  return *this;
}



SnSF::Component& SnSF::Component::rotateLeft(const Permutation& pi){
  int k=pi.size(); // if pi is partial permutation, k<n
  vector<int> vec(k);
  for(int i=0; i<k; i++){
    int r=pi[k-i-1];
    for(int j=0; j<i; j++)
      if(vec[k-j-1]<r) r--;
    vec[k-i-1]=r;
  }
  for(int i=0; i<k; i++)
    rotateLeft(vec[i],0,T.n-k+i+1); // changed 1/7/10
}



SnSF::Component& SnSF::Component::rotateRight(const Permutation& pi){
  int k=pi.size(); // if pi is partial permutation, k<n
  vector<int> vec(k);
  for(int i=0; i<k; i++){
    int r=pi[k-i-1];
    for(int j=0; j<i; j++)
      if(vec[k-j-1]<r) r--;
    vec[k-i-1]=r;
  }
  //cout<<"(";
  for(int i=0; i<k; i++){
    rotateRight(vec[i],0,T.n-k+i+1); // changed 7/15/09
        //cout<<vec[i];
  }
  //cout<<")"<<endl;
}


//SnSF::Component::Component(const ComponentT& _T, ifstream ifs):{
//}


//bool SnSF::Component::writeto(ofstream& ofs){
//}
