#ifndef _SnSFcomponent
#define _SnSFcomponent

#include "SnSF.hpp"
#include "Fmatrix.hpp"
#include "FGmatrix.hpp"
#include "Permutation.hpp"
#include "SnSFcomponentT.hpp"

using namespace SnSF;

class SnSF::Component: public FGmatrix{
public:

  Component(const Component &o): T(o.T), FGmatrix(o) {}
  Component(const ComponentT& _T);
  Component(const ComponentT& _T, const FIELD val);
  //  Component(const ComponentT& _T, const FIELD val): T(_T),FGmatrix(false, _T.Nrows, _T.Ncols, val){};//{cout<<"here COmponent.hpp T(_T) "<<T.str()<<endl;}
  Component(const ComponentT& _T, const Identity& dummy);
  Component(const ComponentT& _T, const Sum<Component, Component>& args);
  Component(const ComponentT& _T, const Product<Component, Component>& args);
  Component(const ComponentT& _T, const Product<Component, Transpose<Component> >& args);
  Component(const ComponentT& _T, const Product<Inverse<Component>, Component>& args);
  Component(const ComponentT& _T, const Product<Component, Inverse<Component> >& args);
  Component(const ComponentT& _T, const Product<Transpose<Component>, Component>& args);
  Component(const ComponentT& _T, const Product<Transpose<Component>, Transpose<Component> >& args);
  Component(const ComponentT& _T, const Product<Component, FIELD >& args);
  Component(const ComponentT& _T, const HadamardProduct<Component, Component>& args);
  Component(const ComponentT& _T, const HadamardDivision<Component, Component>& args);
  Component(const ComponentT& _T, const Fourier& base);
  Component(const ComponentT& _T, ifstream& ifs, const bool binary=0):
    T(_T),FGmatrix(ifs,_T.Nrows,_T.Ncols,binary){};
  virtual ~Component() {}
  
  Component& operator=(const Component& o);
  Component& rotateLeft(const int j, const bool inverse=0, int m=0);
  Component& rotateRight(const int j, const bool inverse=0, int m=0);
  Component& rotateLeft(const Permutation& pi);
  Component& rotateRight(const Permutation& pi);

  Transpose<Component> operator~() const {return Transpose<Component>(*this);}

  FIELD* attab(const Tableau& t1, const Tableau& t2); // why can't I overlad at?

  //bool writeto(ofstream& ofs){};

public:

  const ComponentT& T;

};

inline Component &Component::operator = (const Component& o)
{
  delete a; if(dyadic) delete b;

  if(dyadic) {
    for(int i=0; i<Nrows; i++) a[i]=o.a[i];
    for(int i=0; i<Ncols; i++) b[i]=o.b[i];
  }
  else{
    for(int i=0; i<Nrows*Ncols; i++) a[i]=o.a[i];
  }

  return *this;
}


inline FIELD *Component::attab(const Tableau& t1, const Tableau& t2)
{
  if(dyadic) return NULL;
  int i=0; for(;i<T.Nrows; i++) if(T.rowlabels[i]==t1) break; 
  if(i==T.Nrows) return NULL;
  int j=0; for(;j<T.Ncols; j++) if(T.columnlabels[j]==t2) break; 
  if(j==T.Ncols) return NULL;
  return &a[i*Ncols+j];
}

#endif
