#include "SnSFcomponentT.hpp"
#include "SnSFfourierT.hpp"

#include <cmath>

SnSF::ComponentT::ComponentT(const PartitionB& _shape):
  n(_shape.n()), shape(_shape), YORleftT(NULL), YORrightT(NULL), rowcomplete(0), columncomplete(0), pseudo(0), dyadic(0) {}



SnSF::ComponentT::ComponentT(const OuterProduct<ComponentT> op):
  shape(op.first.shape), n(op.first.n), YORleftT(NULL), YORrightT(NULL), rowcomplete(0), columncomplete(0), pseudo(0), dyadic(0)
{
  const ComponentT& first=op.first;
  const ComponentT& second=op.second;
  // if(first.Ncols==1 && second.Ncols==1 && first.Nrows>1) dyadic=1; //last condition inserted for QAP
  // for(int i=0; i<first.block.size(); i++)
  //  block.push_back(Block(first.block[i].Nrows,second.block[i].Nrows));

  Nrows=first.Nrows;
  for(int i=0; i<Nrows; i++) rowlabels.push_back(first.rowlabels[i]);
  if(first.YORleftT){
    YORleftT=new int[Nrows*(n-1)];
    YORleft1=new double[Nrows*(n-1)];
    YORleft2=new double[Nrows*(n-1)];
    for(int i=0; i<Nrows*(n-1); i++){
      YORleftT[i]=first.YORleftT[i];
      YORleft1[i]=first.YORleft1[i];
      YORleft2[i]=first.YORleft2[i];
    }
  }

  Ncols=second.Nrows;
  for(int i=0; i<Ncols; i++) columnlabels.push_back(second.rowlabels[i]);
  if(second.YORleftT){
    YORrightT=new int[Ncols*(n-1)];
    YORright1=new double[Ncols*(n-1)];
    YORright2=new double[Ncols*(n-1)];
    for(int i=0; i<Ncols*(n-1); i++){
      YORrightT[i]=second.YORleftT[i];
      YORright1[i]=second.YORleft1[i];
      YORright2[i]=second.YORleft2[i];
    }
  }
}



SnSF::ComponentT::ComponentT(const InnerProduct<ComponentT> op):
  shape(op.first.shape), n(op.first.n), YORleftT(NULL), YORrightT(NULL), rowcomplete(0), columncomplete(0), pseudo(0), dyadic(0)
{
  const ComponentT& first=op.first;
  const ComponentT& second=op.second;
  //if(first.Nrows==1 && second.Nrows==1 && first.Ncols>1) dyadic=1; //last condition inserted for QAP
  // for(int i=0; i<first.block.size(); i++)
  //  block.push_back(Block(first.block[i].Nrows,second.block[i].Nrows));

  Nrows=first.Ncols;
  for(int i=0; i<Nrows; i++) rowlabels.push_back(first.columnlabels[i]);
  if(first.YORrightT){
    YORleftT=new int[Nrows*(n-1)];
    YORleft1=new double[Nrows*(n-1)];
    YORleft2=new double[Nrows*(n-1)];
    for(int i=0; i<Nrows*(n-1); i++){
      YORleftT[i]=first.YORrightT[i];
      YORleft1[i]=first.YORright1[i];
      YORleft2[i]=first.YORright2[i];
    }
  }

  Ncols=second.Ncols;
  for(int i=0; i<Ncols; i++) columnlabels.push_back(second.columnlabels[i]);
  if(second.YORrightT){
    YORrightT=new int[Ncols*(n-1)];
    YORright1=new double[Ncols*(n-1)];
    YORright2=new double[Ncols*(n-1)];
    for(int i=0; i<Ncols*(n-1); i++){
      YORrightT[i]=second.YORrightT[i];
      YORright1[i]=second.YORright1[i];
      YORright2[i]=second.YORright2[i];
    }
  }
}



SnSF::ComponentT::ComponentT(const PartitionB& _shape, FourierT* subnode):
  shape(_shape), n(_shape.n()), YORleftT(NULL), YORrightT(NULL), columncomplete(0), Nrows(0), Ncols(0), pseudo(1), dyadic(0)
{
  //cout<<"Shape  "<<shape.str()<<endl;
  //cout<<"n(_shape.n()) "<<n<<endl;
  //getchar();

   if(n==1){

    PartitionB p(1,NULL);
    Tableau t(p);
    rowlabels.push_back(t);
    Nrows=1;
    Ncols=0;
    return;
  }

  for(int i=shape.size()-1; i>=0; i--){
   //cout<<"for loop i "<<i<<endl; 
   if(i==shape.size()-1 || shape[i]>shape[i+1]){
//cout<<"here shape[i] "<<i<<" "<<shape.str()<<endl;
      PartitionB p(shape); p[i]--;
//cout<<"p[i]"<<p[i]<<endl;
//cout<<p.str()<<endl;
//getchar();
      if(p[i]==0) p.pop_back();
      //cout<<p.str()<<endl;
      int subi=(*subnode)[p];
      //cout<<"subi="<<subi<<endl;
      if(subi==-1){
	subnode->push_back(new ComponentT(p,subnode->descendant));
	subi=subnode->size()-1;
      }
      ComponentT& Sub=*(*subnode)[subi];
      sub.push_back(subi);
      Nrows+=Sub.Nrows;
      Ncols+=Sub.Ncols;
      for(int j=0; j<Sub.Nrows; j++){
	Tableau t=Sub.rowlabels[j];
	t.growTo(shape);
	//cout<<t.str()<<endl;
	rowlabels.push_back(t);
      }
      for(int j=0; j<Sub.Ncols; j++){
	Tableau t=Sub.columnlabels[j];
	//cout<<"c"<<t.str()<<endl;
	t.growTo(shape);
	//cout<<"C"<<t.str()<<endl;
	columnlabels.push_back(t);
      }
	
    }
  }
  if(Nrows*Ncols>0){ pseudo=0;
  computeYORleft();
  computeYORright();
  }
    //cout<<"ret"<<endl;
}






SnSF::ComponentT::~ComponentT(){
  if(YORleftT){ delete[] YORleftT; delete[] YORleft1; delete[] YORleft2;} 
  if(YORrightT){ delete[] YORrightT; delete[] YORright1; delete[] YORright2;}
}



int SnSF::ComponentT::YORleft(const int tau, const int i, double& c1, double& c2) const{
//cout<<"YORleftT "<<YORleftT<<endl;
//getchar();
  if (YORleftT){
	  //cout<<"YORleftT "<<endl;
    int index=i*(n-1)+tau-1; 
    c1=YORleft1[index]; 
    c2=YORleft2[index]; 
    return YORleftT[index];
  }

  const Tableau& T=rowlabels[i];
  int row1=T.row[tau];
  int col1=T.col[tau];
  int row2=T.row[tau+1];
  int col2=T.col[tau+1];
  int distance=(col2-row2)-(col1-row1);

  if((row2==row1 && col2==col1+1)||(col2==col1 && row2==row1+1)){
    c1=1.0/distance;
    c2=0;
    return -1;
  }

  Tableau Tdash(T);
  Tdash.row[tau]=T.row[tau+1];
  Tdash.row[tau+1]=T.row[tau];
  Tdash.col[tau]=T.col[tau+1];
  Tdash.col[tau+1]=T.col[tau];
  for(int i=0; i<Nrows; i++)
    if(rowlabels[i]==Tdash){
      c1=1.0/distance;
      c2=sqrt(1-pow(c1,2));
      return i;
    }
  //  cout<<"Alert: rowlabel not found."<<endl;
  c1=1.0/distance;
  c2=0;
  return -1;
}

  

int SnSF::ComponentT::YORright(const int tau, const int i, double& c1, double& c2) const{

  if (YORrightT){
    int index=i*(n-1)+tau-1; 
    c1=YORright1[index]; 
    c2=YORright2[index]; 
    return YORrightT[index];
  }

  const Tableau& T=columnlabels[i];
  int row1=T.row[tau];
  int col1=T.col[tau];
  int row2=T.row[tau+1];
  int col2=T.col[tau+1];
  int distance=(col2-row2)-(col1-row1);

  if((row2==row1 && col2==col1+1)||(col2==col1 && row2==row1+1)){
    c1=1.0/distance;
    c2=0;
    return -1;
  }

  Tableau Tdash(T);
  Tdash.row[tau]=T.row[tau+1];
  Tdash.row[tau+1]=T.row[tau];
  Tdash.col[tau]=T.col[tau+1];
  Tdash.col[tau+1]=T.col[tau];
  for(int i=0; i<Ncols; i++)
    if(columnlabels[i]==Tdash){ // changed 7/16/09
      c1=1.0/distance;
      c2=sqrt(1-pow(c1,2));
      return i;
    }
  //cout<<"Alert: columnlabel not found."<<endl;
  c1=1.0/distance;
  c2=0;
  return -1;
}

  

void SnSF::ComponentT::computeYORleft(){
  if (YORleftT) return;
  int* temp=new int[Nrows*(n-1)];
  YORleft1=new double[Nrows*(n-1)];
  YORleft2=new double[Nrows*(n-1)];
  //cout<<"Nrows="<<Nrows<<endl;
  for(int i=0; i<Nrows; i++){ // for each tableau...
    //cout<<i<<endl;
    for(int j=1; j<n; j++){ // for each transposition (j,j+1)...
      const int index=i*(n-1)+j-1;
      temp[index]=YORleft(j, i, YORleft1[index], YORleft2[index]);
    }
  }
  YORleftT=temp;
//cout<<"YORleftT "<<YORleftT<<endl;
//getchar();
}



void SnSF::ComponentT::computeYORright(){
  if (YORrightT) return;
  int* temp=new int[Ncols*(n-1)];
  YORright1=new double[Ncols*(n-1)];
  YORright2=new double[Ncols*(n-1)];
  //cout<<"Nrows="<<Nrows<<endl;
  for(int i=0; i<Ncols; i++){ // for each tableau...
    //cout<<i<<endl;
    for(int j=1; j<n; j++){ // for each transposition (j,j+1)...
      const int index=i*(n-1)+j-1;
      temp[index]=YORright(j, i, YORright1[index], YORright2[index]);
    }
  }
  YORrightT=temp;
}



string SnSF::ComponentT::str() const{
  ostringstream result;
  result<<"Component "<<shape.str()<<endl;
  //for(int i=0; i<sub.size(); i++) result<<"sub[i] "<<sub[i]<<" size "<<sub.size()<<endl;
  for(int i=0; i<rowlabels.size(); i++)
    result<<"R"<<rowlabels[i].str()<<endl;
  for(int i=0; i<columnlabels.size(); i++)
    result<<"C"<<columnlabels[i].str()<<endl;
  //result<<"("<<block[i].Nrows<<","<<block[i].Ncols<<","<<block[i].index<<")"<<endl;
  return result.str();
}



int SnSF::ComponentT::readfrom(ifstream& ifs){
  //ifs.read(reinterpret_cast<char*>(this),sizeof(ComponentT));
  int t; 
  ifs.read(reinterpret_cast<char*>(&n),sizeof(int));
  shape.readfrom(ifs);
  ifs.read(reinterpret_cast<char*>(&t),sizeof(int)); 
  sub.resize(t); for(int i=0; i<t; i++) ifs.read(reinterpret_cast<char*>(&(sub[i])),sizeof(int));
  ifs.read(reinterpret_cast<char*>(&pseudo),sizeof(bool));

  ifs.read(reinterpret_cast<char*>(&Nrows),sizeof(int)); 
  rowlabels.resize(Nrows); for(int i=0; i<Nrows; i++) rowlabels[i].readfrom(ifs); 
  ifs.read(reinterpret_cast<char*>(&rowcomplete),sizeof(bool));
  ifs.read(reinterpret_cast<char*>(&YORleftT),sizeof(int*));
  ifs.read(reinterpret_cast<char*>(&YORleft1),sizeof(double*));
  ifs.read(reinterpret_cast<char*>(&YORleft1),sizeof(double*));  
  if(YORleftT){
    YORleftT=new int[Nrows*(n-1)];
    YORleft1=new double[Nrows*(n-1)];
    YORleft2=new double[Nrows*(n-1)];
    ifs.read(reinterpret_cast<char*>(YORleftT),sizeof(int)*Nrows*(n-1));
    ifs.read(reinterpret_cast<char*>(YORleft1),sizeof(double)*Nrows*(n-1));
    ifs.read(reinterpret_cast<char*>(YORleft2),sizeof(double)*Nrows*(n-1));
  }else{YORleft1=NULL; YORleft2=NULL;}

  ifs.read(reinterpret_cast<char*>(&Ncols),sizeof(int));
  columnlabels.resize(Ncols); for(int i=0; i<Ncols; i++) columnlabels[i].readfrom(ifs); 
  ifs.read(reinterpret_cast<char*>(&columncomplete),sizeof(bool));
  ifs.read(reinterpret_cast<char*>(&YORrightT),sizeof(int*));
  ifs.read(reinterpret_cast<char*>(&YORright1),sizeof(double*));
  ifs.read(reinterpret_cast<char*>(&YORright1),sizeof(double*));
  if(YORrightT){
    YORrightT=new int[Ncols*(n-1)];
    YORright1=new double[Ncols*(n-1)];
    YORright2=new double[Ncols*(n-1)];
    ifs.read(reinterpret_cast<char*>(YORrightT),sizeof(int)*Ncols*(n-1));
    ifs.read(reinterpret_cast<char*>(YORright1),sizeof(double)*Ncols*(n-1));
    ifs.read(reinterpret_cast<char*>(YORright2),sizeof(double)*Ncols*(n-1));
  }{YORright1=NULL; YORright2=NULL;}
  
}


int SnSF::ComponentT::writeto(ofstream& ofs) const{
  //ofs.write(reinterpret_cast<const char*>(this),sizeof(ComponentT));
  int t;
  ofs.write(reinterpret_cast<const char*>(&n),sizeof(int));
  shape.writeto(ofs);
  t=sub.size(); ofs.write(reinterpret_cast<char*>(&t),sizeof(int));
  for(int i=0; i<t; i++) ofs.write(reinterpret_cast<const char*>(&(sub[i])),sizeof(int));
  ofs.write(reinterpret_cast<const char*>(&pseudo),sizeof(bool));

  ofs.write(reinterpret_cast<const char*>(&Nrows),sizeof(int));
  for(int i=0; i<Nrows; i++) rowlabels[i].writeto(ofs);
  ofs.write(reinterpret_cast<const char*>(&rowcomplete),sizeof(bool));
  ofs.write(reinterpret_cast<const char*>(&YORleftT),sizeof(int*));
  ofs.write(reinterpret_cast<const char*>(&YORleft1),sizeof(double*));
  ofs.write(reinterpret_cast<const char*>(&YORleft1),sizeof(double*));
  if(YORleftT){
    ofs.write(reinterpret_cast<const char*>(YORleftT),sizeof(int)*Nrows*(n-1));
    ofs.write(reinterpret_cast<const char*>(YORleft1),sizeof(double)*Nrows*(n-1));
    ofs.write(reinterpret_cast<const char*>(YORleft2),sizeof(double)*Nrows*(n-1));
  }

  ofs.write(reinterpret_cast<const char*>(&Ncols),sizeof(int));
  for(int i=0; i<Ncols; i++) columnlabels[i].writeto(ofs);
  ofs.write(reinterpret_cast<const char*>(&columncomplete),sizeof(bool));
  ofs.write(reinterpret_cast<const char*>(&YORrightT),sizeof(int*));
  ofs.write(reinterpret_cast<const char*>(&YORright1),sizeof(double*));
  ofs.write(reinterpret_cast<const char*>(&YORright1),sizeof(double*));
  if(YORrightT){
    ofs.write(reinterpret_cast<const char*>(YORrightT),sizeof(int)*Ncols*(n-1));
    ofs.write(reinterpret_cast<const char*>(YORright1),sizeof(double)*Ncols*(n-1));
    ofs.write(reinterpret_cast<const char*>(YORright2),sizeof(double)*Ncols*(n-1));
  }
  
}
