#include "SnSFfourier.hpp"
#include "SnSFfourierT.hpp"
#include "SnSFcomponent.hpp"
#include "SnSFcomponentT.hpp"
#include "SnSFfunction.hpp"
#include "SnSFsfunction.hpp"
#include "AssignmentMatrix.hpp"
#include <typeinfo>

#include <cassert>

#ifdef _branchCounter
extern ScalarCounter branchCounter;
#endif


SnSF::Fourier::Fourier(const Fourier& o) :
  T(o.T)
{
  for(int i = 0; i < o.size(); i++) {
    push_back(new Component(*o[i]));
  }
}

SnSF::Fourier::Fourier(const FourierT& _T) :
  T(_T)
{
  for(int i = 0; i < T.size(); i++) {
    if(!T[i]->pseudo) {
      push_back(new Component(*T[i]));
    }
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const FIELD val) :
  T(_T)
{
  for(int i = 0; i < T.size(); i++) {
    if(!T[i]->pseudo) {
      push_back(new Component(*T[i],val));
    }
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const Identity& dummy, const int C) :
  T(_T)
{
  for(int i = 0; i < T.size() && i != C; i++) {
    if(!T[i]->pseudo) {
      push_back(new Component(*T[i],dummy));
    }
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const Sum<Fourier, Fourier>& args) :
  T(_T)
{
  assert(T.size() == args.left.size());
  assert(T.size() == args.right.size());

  for(int i = 0; i < T.size(); i++) {
    push_back(new Component(*T[i],Sum<Component,Component>(*args.left[i],*args.right[i])));
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const Product<Fourier, Fourier>& args) :
  T(_T)
{
  assert(T.size() == args.left.size());
  assert(T.size() == args.right.size());

  for(int i = 0; i < T.size(); i++) {
    push_back(new Component(*T[i],Product<Component,Component>(*args.left[i],*args.right[i])));
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const Product<Fourier, Transpose<Fourier> >& args) : 
  T(_T)
{
  assert(T.size() == args.left.size());
  assert(T.size() == args.right.val.size());

  for(int i = 0; i < T.size(); i++) {
    push_back(new Component(*T[i],Product<Component,Transpose<Component> >(*args.left[i],*args.right.val[i])));
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const Product<Fourier, Inverse<Fourier> >& args) : 
  T(_T)
{
  assert(T.size() == args.left.size());
  assert(T.size() == args.right.val.size());

  for(int i = 0; i < T.size(); i++) {
    push_back(new Component(*T[i],Product<Component,Inverse<Component> >(*args.left[i],*args.right.val[i])));
  }
}

SnSF::Fourier::Fourier(
  const FourierT &_T,
  const Product<Inverse<Fourier>, Fourier> &args) :
    T(_T)
{
  assert(T.size() == args.left.val.size());
  assert(T.size() == args.right.size());

  for(int i = 0; i < T.size(); i++) {
    push_back(new Component(*T[i], Product<Inverse<Component>, Component>(*args.left.val[i], *args.right[i])));
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const Product<Transpose<Fourier>, Fourier>& args) : T(_T)
{
  assert(T.size() == args.left.val.size());
  assert(T.size() == args.right.size());

  for(int i = 0; i < T.size(); i++) {
    push_back(new Component(*T[i],Product<Transpose<Component>,Component>(*args.left.val[i],*args.right[i])));
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const Product<Transpose<Fourier>, Transpose<Fourier> >& args) : T(_T)
{
  assert(T.size() == args.left.val.size());
  assert(T.size() == args.right.val.size());

  for(int i = 0; i < T.size(); i++) {
    push_back(new Component(*T[i],Product<Transpose<Component>,Transpose<Component> >(*args.left.val[i],*args.right.val[i])));
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const Product<Fourier, FIELD>& args) : T(_T)
{
  assert(T.size() == args.left.size());

  for(int i = 0; i < T.size(); i++) {
    push_back(new Component(*T[i],Product<Component,FIELD>(*args.left[i],args.right)));
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const HadamardProduct<Fourier, Fourier> &args) : T(_T)
{
  assert(T.size() == args.left.size());
  assert(T.size() == args.right.size());

  for(int i = 0; i < T.size(); i++) {
    HadamardProduct<Component, Component> expr(*args.left[i], *args.right[i]);
    push_back(new Component(*T[i], expr));
  }
}

SnSF::Fourier::Fourier(const FourierT& _T, const HadamardDivision<Fourier, Fourier>& args) : T(_T)
{
  assert(T.size() == args.left.size());
  assert(T.size() == args.right.size());

  for(int i = 0; i < T.size(); i++) {
    HadamardDivision<Component, Component> expr(*args.left[i], *args.right[i]);
    push_back(new Component(*T[i], expr));
  }
}

SnSF::Fourier::~Fourier()
{
  for(int i = 0; i < size(); i++) {
    delete at(i);
  }
}




bool SnSFfourierPaircomparison(pair<int,double> a, pair<int,double> b){ return(a.second>b.second); }



SnSF::Fourier::Fourier(const FourierT& _T, const int k, int* I): T(_T){

 if(k==0){
    for(int i=0; i<T.size(); i++) 
      if(!T[i]->pseudo) push_back(new Component(*T[i],0));
    (*this)[0]->at(0,0)=1;
    return;
  }
  for(int i=1; i<k ;i++)
    if(I[i]>I[0]) I[i]--;
  Fourier sub(*T.descendant,k-1,I+1);
  for(int i=0; i<T.size(); i++) 
    if(!T[i]->pseudo) push_back(new Component(*T[i],sub));
  rotateLeft(I[0]);
}


// seems like it will only work if pseudos are at end
SnSF::Fourier::Fourier(const FourierT& _T, const Fourier& base): T(_T){
  const int n = T.n;
  if(n==base.T.n+1){
    for(int i=0; i<T.size(); i++) 
      if(!T[i]->pseudo) push_back(new Component(*T[i],base));
    return;
  }
  if(n==base.T.n-1){
    for(int i=0; i<T.size(); i++){
      if(!T[i]->pseudo) {
	push_back(new Component(*T[i],0));
      }
    }
  
    for(int ai=0; ai<base.size(); ai++){
      const Component& a=*(base[ai]);
      int rowoffset=0;
      int columnoffset=0;
      for(int subi=0; subi<a.T.sub.size(); subi++){
	ComponentT& subT=*T[a.T.sub[subi]];
	int nrows=subT.Nrows;
	int ncols=subT.Ncols;
	if(!subT.pseudo){
	  Component& b=*at(a.T.sub[subi]);	
	  double multiplier=((double)a.Nrows)/((double)b.Nrows*((double)(n+1)));
	  
	  //b.dyadic = 0;//This is a hack by DP
	  if(b.dyadic){// can only be dyadic if multiplicity is 1 and parent is dyadic
	    for(int i=0; i<nrows; i++) b.a[i]=a.a[i+rowoffset]*multiplier;
	    for(int i=0; i<ncols; i++) b.b[i]=a.b[i+columnoffset];
	    }else{
	    for(int i=0; i<nrows; i++)
	      for(int j=0; j<ncols; j++)
		b(i,j)+=a(i+rowoffset,j+columnoffset)*multiplier;
	    }
	}
	rowoffset+=nrows;
	columnoffset+=ncols;
      }    
    }
  }
}




SnSF::Fourier::Fourier(const Function& f): T(f.T) 
{
  const int n=f.T.n;
  const double order=f.T.order;

  for(int i=0; i<T.size(); i++)
    if(!T[i]->pseudo) push_back(new Component(*T[i],0));
  
  if(order==1){
    (*this)[0]->at(0,0)=f.f[0];
    return;
  }

  int suborder=((int)order)/n;  
  for(int j=1; j<=n; j++){
    Function subf(*(T.descendant),f.f+(n-j)*suborder);
    Fourier sub(subf);
    //cout<<"sub:"<<sub.str()<<endl;
    Fourier subtilde(T,sub);
    //cout<<"subtilde:"<<subtilde.str()<<endl;
    subtilde.rotateLeft(j);
    (*this)+=subtilde;
  }
  //cout<<str()<<endl;
  
}


// SnSF::Fourier::Fourier(const FourierT& _T, const Fourier& base): T(_T)
// {
// }


SnSF::Fourier::Fourier(const Sfunction& f): T(f.T) 
{
  const int n=f.T.n;
  const int width=f.width;

  // cout<<n<<endl;

  for(int i=0; i<T.size(); i++)
    if(!T[i]->pseudo) push_back(new Component(*T[i],0));
  
  if(f.T.order==1){
    if(f.size==1)
      (*this)[0]->at(0,0)=f.f[0];
    else{
      cout<<"alert "<<n<<" "<<f.size<<endl;
      (*this)[0]->at(0,0)=0;
    }
    return;
  }
  
  int offset=0;
  for(int j=n; j>=1; j--){
    int i=offset;
    for(; i<f.size; i++)
      if(f.index[i*width+n-2]<j) break;
    if(i>offset){
      Sfunction subf(f,offset,i-offset);
      Fourier sub(subf);
      Fourier subtilde(T,sub);
      subtilde.rotateLeft(j);
      (*this)+=subtilde;
    }
    offset=i;
  }
}





string SnSF::Fourier::str() const{
  ostringstream result;
  for(int i=0; i<size(); i++) result<<at(i)->str()<<endl;
  return result.str();
}


ostream& operator<<(ostream& stream, const SnSF::Fourier& o){
  //cout << "this size is " << o.size() << endl;
  for(int i=0; i<o.size(); i++) stream<<*o.at(i)<<endl;
  return stream;
}

void SnSF::Fourier::printSizes(std::ostream &stream) const
{
  int n = size();
  cout<<n<<' ';

  for(int i = 0; i < n; i++) {
    stream<<(*this)[i]->Nrows<<'x'<<(*this)[i]->Ncols;
    if((*this)[i]->dyadic) {
      stream<<" (dyadic)";
    }

    if(T[i]->pseudo) {
      cout<<" (pseudo)";
    }

    if(i != n - 1) {
      stream<<' ';
    }
  }
}

//bool SnSF::Fourier::readfrom(ifstream& ifs){
//}



//bool SnSF::Fourier::writeto(ofstream& ofs){
//}


///////////////////////////// Modifiers ////////////////////////

const SnSF::Fourier &SnSF::Fourier::operator = (const Fourier& o)
{
  assert(size() == o.size());

  for(int i = 0; i < size(); i++) {
    *((*this)[i]) = *(o[i]); 
  }

  return *this;
}

const SnSF::Fourier &SnSF::Fourier::operator += (const Fourier& o)
{
  assert(size() == o.size());

  for(int i = 0; i < size(); i++) {
    *((*this)[i]) += *(o[i]);
  }

  return *this;
}

const SnSF::Fourier &SnSF::Fourier::operator -= (const Fourier& o)
{
  assert(size() == o.size());

  for(int i = 0; i < size(); i++) {
    *((*this)[i]) -= *(o[i]); 
  }

  return *this;
}

const SnSF::Fourier &SnSF::Fourier::operator *= (const FIELD o)
{
  for(int i = 0; i < size(); i++) {
    *((*this)[i]) *= o;
  }

  return *this;
}

const SnSF::Fourier &SnSF::Fourier::rotateLeft(const int j, const bool inverse, const int m)
{
  for(vector<Component*>::iterator it = begin(); it != end(); it++) {
    (*it)->rotateLeft(j, inverse, m); 
  }

  return *this;
}

const SnSF::Fourier &SnSF::Fourier::rotateRight(const int j, const bool inverse, const int m)
{
  for(vector<Component*>::iterator it = begin(); it != end(); it++) {
    (*it)->rotateRight(j,inverse,m);
  }

  return *this;
}

const SnSF::Fourier &SnSF::Fourier::rotateLeft(const Permutation& pi)
{
  for(vector<Component*>::iterator it = begin(); it != end(); it++) {
    (*it)->rotateLeft(pi);
  }

  return *this;
}

const SnSF::Fourier &SnSF::Fourier::rotateRight(const Permutation& pi)
{
  for(vector<Component*>::iterator it = begin(); it != end(); it++) {
    (*it)->rotateRight(pi);
  }

  return *this;
}

bool SnSF::Fourier::isNaN() const
{
  for(int i = 0; i < size(); i++) {
    if(at(i)->isNaN()) {
      return true;
    }
  }

  return false;
}
