#ifndef _SnSFfourier
#define _SnSFfourier

#include "SnSF.hpp"
#include "SnSFfourierT.hpp"
#include "SnSFcomponent.hpp"
#include "Permutation.hpp"
#include "InheritableVector.hpp"

#include <cassert>

using namespace SnSF;

class SnSF::Fourier: public InheritableVector<Component*> {
public:

  Fourier(const Fourier& o);
  Fourier(const FourierT& _T);
  Fourier(const bool dummy, const FourierT& _T): T(_T){};
  Fourier(const FourierT& _T, const FIELD val);
  Fourier(const FourierT& _T, const Identity& dummy, const int C=-1);
  Fourier(const FourierT& _T, const int k, int* I);
  Fourier(const FourierT& _T, const Fourier& base);
  Fourier(const FourierT& _T, const Sum<Fourier, Fourier>& args);
  Fourier(const FourierT& _T, const Product<Fourier, Fourier>& args);
  Fourier(const FourierT& _T, const Product<Fourier, Transpose<Fourier> >& args);
  Fourier(const FourierT& _T, const Product<Transpose<Fourier>, Fourier>& args);
  Fourier(const FourierT& _T, const Product<Transpose<Fourier>, Transpose<Fourier> >& args);
  //DP inverse 
  Fourier(const FourierT& _T, const Product<Fourier, Inverse<Fourier> >& args);
  Fourier(const FourierT& _T, const Product<Inverse<Fourier>, Fourier>& args);
  //
  Fourier(const FourierT& _T, const Product<Fourier, FIELD>& args);
  Fourier(const FourierT& _T, const HadamardProduct<Fourier, Fourier>& args);
  Fourier(const FourierT& _T, const HadamardDivision<Fourier, Fourier>& args);
  Fourier(const Function& f);
  //Fourier(const Transform& _T, const Fourier& base);
  Fourier(const Sfunction& f);
  virtual ~Fourier();

  const Fourier &operator  = (const Fourier& o);
  const Fourier &operator += (const Fourier& o);
  const Fourier &operator -= (const Fourier& o);
  const Fourier &operator *= (const FIELD o);
  const Fourier &rotateLeft(const int j, const bool inverse=0, const int m=0);
  const Fourier &rotateRight(const int j, const bool inverse=0, const int m=0);
  const Fourier &rotateLeft(const Permutation& pi);
  const Fourier &rotateRight(const Permutation& pi);

  Transpose<Fourier> operator~() const {return Transpose<Fourier>(*this);}

  Component *operator[] (int i) const {
    assert(i >= 0 && i < size());
    return at(i);
  };
 
  string str() const;
  void printSizes(std::ostream &) const;

  /**
   * Whether any of the components has NaN entries.
   */
  bool isNaN() const;

public:
  
  const FourierT& T; 

};

ostream& operator<<(ostream& stream, const SnSF::Fourier& o);


#endif
