#include "SnSFfunction.hpp"

#include "SnSFfourier.hpp"
#include "SnSFfourierT.hpp"
#include "SnSFcomponent.hpp"
#include "SnSFsfunction.hpp"

#include <cmath>
#include <cassert>

SnSF::Function::Function(const Function& o): T(o.T), protect(0){
  getchar();
  f=new FIELD[(int)T.order]; for(int i=0; i<T.order; i++) f[i]=o.f[i];
}

SnSF::Function::Function(const SnSF::Sfunction& o): T(o.T), protect(0){
  f=new FIELD[(int)T.order];
  const int n=o.T.n;
  const int width=o.width;
  for(int i=0; i<T.order; i++) f[i]=0;
  for(int i=0; i<o.size; i++){
    int offset=0;
    int mult=1;
    for(int j=1; j<n; j++){
      mult*=j;
      offset+=(j+1-o.index[i*width+j-1])*mult;
    }
    f[offset]=o.f[i];
  }
}

SnSF::Function::Function(const Transform& transform) :
  T(transform.top()),
  protect(0)
{
  f = new FIELD[(int)T.order];
}

SnSF::Function::Function(const Transform& transform, const FIELD val): T(transform.top()), protect(0){
  f=new FIELD[(int)T.order]; for(int i=0; i<T.order; i++) f[i]=val;
}

SnSF::Function::Function(const FourierT& _T): T(_T), protect(0){
  f=new FIELD[(int)T.order];
}

SnSF::Function::Function(const FourierT& _T, const FIELD val): T(_T), protect(0){
  f=new FIELD[(int)T.order]; for(int i=0; i<T.order; i++) f[i]=val; 
}

SnSF::Function::Function(const FourierT& _T, FIELD* _f): T(_T), protect(1) {
  f=_f;
}

SnSF::Function::~Function(){
  if(!protect) delete[] f;
}




SnSF::Function::Function(const Fourier& F): T(F.T), protect(0){
  f=new FIELD[(int)T.order];
  Function me(F,f);
}



SnSF::Function::Function(const Fourier& F, FIELD* _f): T(F.T), protect(1){
  f=_f;
  const int n=T.n;
  const int order=(int)T.order;
  const int suborder=order/n;
  //cout<<n<<endl;

  if(order==1){
    //cout<<"a"<<endl;
    f[0]=F[0]->at(0,0);
    //cout<<f[0]<<endl;
    return;
  }

  for(int j=1; j<=n; j++){
    //cout<<n<<j<<endl;
    //cout<<"A"<<endl;
    Fourier subtilde(F);
    //cout<<"B"<<endl;
    //cout<<F.str()<<endl;
    //cout<<subtilde.str()<<endl;
    subtilde.rotateLeft(j,1);
    //cout<<"C"<<endl;
    Fourier sub(*F.T.descendant,subtilde);
    //cout<<"D"<<endl;
    Function subf(sub,f+(n-j)*suborder);
  }
}



SnSF::Function& SnSF::Function::randomize(){
  for(int i=0; i<T.order; i++) f[i]=((double) rand())/((double)RAND_MAX+1);
  return *this;
}



string SnSF::Function::str() const{
  ostringstream result;
  cout << "Order is " << T.order << endl;
  for(int i=0; i<T.order; i++)
    if (abs(f[i])<0.00001) result<<"0"<<endl;
    else result<<f[i]<<endl;
  return result.str();
}


ostream& operator<<(ostream& stream, const SnSF::Function& o){
  for(int i=0; i<o.T.order; i++)
    if (abs(o.f[i])<0.00001) stream<<"0"<<endl;
    else stream<<o.f[i]<<endl;
  return stream;
}

#ifndef INFINITY
#define INFINITY (1.0/0.0)
#endif

FIELD SnSF::Function::eval(const Permutation &perm) const
{
  return f[perm2ind(perm)];
}

FIELD &SnSF::Function::eval(const Permutation &perm)
{
  return f[perm2ind(perm)];
}

int SnSF::Function::minind() const
{
  FIELD min = INFINITY;
  int ind = -1;

  for(int i = 0; i < T.order; i++) {
    if(f[i] < min) {
      min = f[i];
      ind = i;
    }
  }

  return ind;
}

int SnSF::Function::maxind() const
{
  FIELD max = -INFINITY;
  int ind = -1;

  for(int i = 0; i < T.order; i++) {
    if(f[i] > max) {
      max = f[i];
      ind = i;
    }
  }

  return ind;
}

FIELD SnSF::Function::minimum() const
{
  return f[minind()];
}

Permutation SnSF::Function::minimumPermutation() const
{
  return ind2perm(minind());
}

FIELD SnSF::Function::maximum() const
{
  return f[maxind()];
}

Permutation SnSF::Function::maximumPermutation() const
{
  return ind2perm(maxind());
}

static void printVec(int n, const FIELD *f)
{
  for(int i = 0; i < n; i++) {
    if(f[i] < 1e-6) {
      cout<<"0 ";
    }
    else {
      cout<<f[i]<<' ';
    }
  }
  cout<<endl;
}

Permutation SnSF::Function::ind2perm(int ind) const
{
  int n = T.n;
  vector<int> cclist(n);

  int order = 1;

  for(int i = n; i >= 1; i--) {

    int child = (i+1) - (ind % order * (i+1)) / order;
    ind = ind - ind % order;

    if(i != n) {
      cclist[i] = child;
      //cout<<i<<' '<<ind<<' '<<order<<' '<<child<<endl;
    }
    order *= i;
  }
  cclist[0] = 1;

  return Permutation(CClist(cclist)).inverse();
}

int SnSF::Function::perm2ind(const Permutation &perm) const
{
  int n = perm.size();
  assert(n == T.n);

  vector<int> cclist = perm.inverse().cclist();
  assert(n == cclist.size());

  // Artifically descend the tree following branches corresponding to perm
  int order = T.order;
  int ind = 0;
  for(int i = 1; i <= n; i++) {
    int child = cclist[i-1];

    assert(order % i == 0);
    order /= i;

    ind += (i - child) * order;

    //cout<<i<<' '<<ind<<' '<<order<<' '<<child<<endl;
  }

  assert(order == 1);
  assert(ind < T.order);
  return ind;
}
