#ifndef _SnSFfunction
#define _SnSFfunction

#include "SnSF.hpp"
#include "SnSFtransform.hpp"
#include "Permutation.hpp"

using namespace std;



class SnSF::Function{
public:

  Function(const Function& o);
  Function(const Sfunction& o);

  Function(const Transform& _transfm);
  Function(const Transform& _transfm, const FIELD val);
  Function(const FourierT& _T);
  Function(const FourierT& _T, const FIELD val);
  Function(const FourierT& _T, FIELD* _f);

  Function(const Fourier& f);

  Function(const Fourier& F, FIELD* _f);
  virtual ~Function();


  Fourier* fft();

  Function& randomize();
  string str() const;

  /**
   * Find the minimum of the function.
   */
  FIELD minimum() const;

  /**
   * Find in real-space the permutation that minimizes the function.
   */
  Permutation minimumPermutation() const;

  /**
   * Find the maximum of the function.
   */
  FIELD maximum() const;

  /**
   * Find in real-space the permutation that maximizes the function.
   */
  Permutation maximumPermutation() const;

  /**
   * Find the function value corresponding to a specific permutation.
   */
  FIELD eval(const Permutation &perm) const;
  FIELD &eval(const Permutation &perm);

public:

  const FourierT& T;
  bool protect;
  FIELD* f;

  //private:
  int minind() const;
  int maxind() const;

  /**
   * Generate permutation corresponding to a given entry in the @c f vector.
   */
  Permutation ind2perm(int ind) const;

  /**
   * Generate index into @c f  for a given permuation
   */
  int perm2ind(const Permutation &) const;
};


ostream& operator<<(ostream& stream, const SnSF::Function& o);



#endif
