#include "SnSFsfunction.hpp"

#include "SnSFfourier.hpp"
#include "SnSFfourierT.hpp"
#include "SnSFcomponent.hpp"
#include "SnSFfunction.hpp"



SnSF::Sfunction::Sfunction(const Sfunction& o): T(o.T), width(o.width), size(o.size), protect(0){
  index=new int[size*width]; for(int i=0; i<size*width; i++) index[i]=o.index[i];
  f=new FIELD[size]; for(int i=0; i<size; i++) f[i]=o.f[i];
}



SnSF::Sfunction::Sfunction(const Sfunction& o, const int offset, const int _size): 
  T(*(o.T.descendant)), 
  width(o.width), 
  size(_size),
  protect(1)
{
  index=o.index+offset*width;
  f=o.f+offset;
}



SnSF::Sfunction::Sfunction(const Transform& transform, const int _size): 
  T(transform.top()), 
  width(transform.top().n-1),
  size(_size),
  protect(0)
{
  index=NULL; f=NULL; 
  if(size>0){
    index=new int[size*width];
    f=new FIELD[size];
  }
}



SnSF::Sfunction::Sfunction(const FourierT& _T, const int _size): 
  T(_T), 
  width(T.n), 
  size(_size), 
  protect(0)
{
  index=NULL; f=NULL; 
  if(size>0){
    index=new int[size*width];
    f=new FIELD[size];
  }
}



SnSF::Sfunction::Sfunction(const Function& o): T(o.T), width(o.T.n-1), protect(0){
  const int n=T.n;
  const int order=(int)T.order;
  size=0;
  for(int i=0; i<order; i++) if(o.f[i]!=0) size++;
  index=new int[size*width];
  f=new FIELD[size];
  int c=0;
  for(int i=0; i<order; i++) 
    if(o.f[i]!=0){
      f[c]=o.f[i];
      int ix=i;
      for(int j=1; j<n; j++){
	int a=(int) ix%(j+1);
	ix=(int)(ix-a)/(j+1);
	index[c*width+j-1]=j+1-a;
      }
      c++;
    }
}

SnSF::Sfunction::~Sfunction(){
  if(!protect) {delete[] f; delete[] index;}
}




SnSF::Sfunction::Sfunction(const Fourier& F): T(F.T), width(F.T.n), size(0), protect(0){
  // Sfunction me(F,f);
}



SnSF::Sfunction::Sfunction(const Fourier& F, FIELD* _f): T(F.T), width(F.T.n), protect(1){
  f=_f;
  const int n=T.n;
  const int order=(int)T.order;
  const int suborder=order/n;
  //cout<<n<<endl;

  if(order==1){
    //cout<<"a"<<endl;
    f[0]=F[0]->at(0,0);
    //cout<<f[0]<<endl;
    return;
  }

  for(int j=1; j<=n; j++){
    //cout<<n<<j<<endl;
    //cout<<"A"<<endl;
    Fourier subtilde(F);
    //cout<<"B"<<endl;
    //cout<<F.str()<<endl;
    //cout<<subtilde.str()<<endl;
    subtilde.rotateLeft(j,1);
    //cout<<"C"<<endl;
    Fourier sub(*F.T.descendant,subtilde);
    //cout<<"D"<<endl;
    Sfunction subf(sub,f+(n-j)*suborder);
  }
}





string SnSF::Sfunction::str() const{
  ostringstream result;
  for(int i=0; i<size; i++){
    result<<"[";
    for(int j=0; j<width; j++) result<<index[i*width+j];
    result<<"] ";
    result<<f[i]<<endl;
  }
  //cout << "done result.str" << endl;
  return result.str();
}
