import cairo
from math import *
import sys


def loadPoints(ptfile):
    p = []
    for line in file(ptfile):
        line = line.strip()
        if line == '':
            continue

        line = [x for x in line.split() if x != '']
        line = float(line[0]),float(line[1])

        p.append(line)

    p = [p[i-1] for i in nodes]
    return p

def plotPoints(outpath, ptfile, imgfile, graph = None):
    # Load graph
    if graph != None:
        G = []
        for row in file(graph):
            row = row.strip()
            if row == '':
                continue

            row = [float(col) for col in row.split(' ') if col != '']
            G.append(row)
        graph = G

    # Load the image
    img = cairo.ImageSurface.create_from_png(imgfile)

    width = img.get_width()
    height = img.get_height()

    # Create output
    surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
    cr = cairo.Context(surf)

    # Blit image
    cr.set_source_surface(img, 0, 0)
    cr.paint()

    points = loadPoints(ptfile)

    # Draw graph
    cr.set_source_rgb(1, 0, 0)
    cr.set_line_width(max(cr.device_to_user_distance(2, 2))/2)
    if graph != None:
        for i in range(len(points)):
            for j in range(len(points)):
                if i == j or graph[i][j] == 0:
                    continue


                cr.move_to(points[i][0], points[i][1])
                cr.line_to(points[j][0], points[j][1])
                cr.stroke()

    # Draw points
    for num, p in enumerate(points):
        cr.set_source_rgb(1, 0, 0)
        cr.arc(p[0], p[1], max(cr.device_to_user_distance(2, 2)), 0, 2 * pi)
        cr.fill()

        cr.set_source_rgb(0, 1, 0)
        cr.translate(p[0], p[1])
        cr.show_text(str(num + 1))
        cr.translate(-p[0], -p[1])
    
    # Write
    surf.write_to_png(outpath)
    surf.finish()

if __name__ == '__main__':
    outpath = sys.argv[1]
    ptfile = sys.argv[2]
    imgfile = sys.argv[3]
    graph = sys.argv[4]
    if graph.lower() == 'none':
        graph = None

    nodes = map(int, sys.argv[5:])

    plotPoints(outpath, ptfile, imgfile, graph)
