#ifndef _Graph
#define _Graph


#include "Fmatrix.hpp"
#include "SnSF.hpp"
#include "SnSFtransform.hpp"
#include "Permutation.hpp"

class Graph: public Fmatrix{
public:

  Graph(const int n);
  Graph(const int n, const FIELD val);
  Graph(const char* s1, ...);

  Graph(const Graph& o);
  Graph(const Fmatrix &F);

  Graph(const string filename) : Fmatrix(filename)
  {
    assert(Nrows == Ncols);
  };

  Graph(ifstream& ifs, const int _Nrows, const int _Ncols) : 
    Fmatrix(ifs,_Nrows,_Ncols)
  {};

  const Graph &operator = (const Graph &);
  virtual ~Graph();

  
  SnSF::Function* function(const SnSF::Transform& _transfm) const;
  SnSF::Fourier* Fourier(const SnSF::Transform& _transfm) const;

  /**
   * Permute the nodes of the graph.
   *
   * @return Permuted graph
   */
  Graph permute(const Permutation& p) const;

  /**
   * "Negate" the weights of a graph.
   *
   * Returns a graph with weights
   *
   * @[
   *   w'_{ij} = \left( \max_{ij} w_{ij} \right) - w_{ij}
   * @]
   */
  Graph pseudoNegate();

  /**
   * Select a subset of the nodes.
   */
  Graph subgraph(const std::vector<int> &nodes) const;

  class Cycle;
  class Circuit;
  class Chain;
  class RandomSimple;
  class RandomBinary;
  class RandomWeighted;
  class RandomNedges;

  friend std::ostream &operator << (std::ostream &, const Graph &);
};



class Graph::Cycle: public Graph{
public:
  Cycle(const int n, int m=0);
};


class Graph::Circuit: public Graph{
public:
  Circuit(const int n, int m=0);
};


class Graph::Chain: public Graph{
public:
  Chain(const int n, int m=0);
};


class Graph::RandomSimple: public Graph{
public:
  RandomSimple(const int n, const double v=0.5);
};


class Graph::RandomBinary: public Graph{
public:
  RandomBinary(const int n, const double v=0.5);
};


class Graph::RandomWeighted: public Graph{
public:
  RandomWeighted(const int n, const double thresh=0);
};

/**
 * A random 0-1 graph generated by specifying the number of edges.
 *
 * @note This will be an undirected (and thus symmetric) graph.
 */
class Graph::RandomNedges: public Graph
{
public:
  RandomNedges(int n, int nedges);
};

std::ostream &operator << (std::ostream &, const Graph &);

#endif
