#include "AssignmentMatrix.hpp"



AssignmentMatrix::AssignmentMatrix(const int _Nrows, const int _Ncols): 
  Fmatrix(_Nrows,_Ncols), 
  n(_Nrows),
  yx(_Nrows),
  yy(_Nrows),
  assignedx(_Nrows,-1),
  assignedy(_Nrows,-1),
  foundx(_Nrows),
  foundy(_Nrows),
  Nassignments(0)
{
};

AssignmentMatrix::AssignmentMatrix(const Fmatrix &F) :
  Fmatrix(F),
  n(F.Nrows),
  yx(F.Nrows),
  yy(F.Nrows),
  assignedx(F.Nrows,-1),
  assignedy(F.Nrows,-1),
  foundx(F.Nrows),
  foundy(F.Nrows),
  Nassignments(0)
{
}


AssignmentMatrix::~AssignmentMatrix(){
}



double AssignmentMatrix::KuhnMunkres(){
  double max=0; for(int i=0; i<Nrows*Ncols; i++) if (max<a[i]) max=a[i];
  yx.fill(max);  yy.fill(max);
  Nassignments=0;
  while(Nassignments<n){
    Augment();
    double delta=-1;
    for(int x=0; x<n; x++)
      for(int y=0; y<n; y++)
	if(foundx(x) && !foundy(y) && (delta==-1 || -at(x,y)+yx(x)+yy(y)<delta))
	  delta=-at(x,y)+yx(x)+yy(y);

    if(delta!=-1){
      for(int x=0; x<n; x++) if(foundx(x)) yx(x)-=delta;
      for(int y=0; y<n; y++) if(foundy(y)) yy(y)+=delta;
    } 
    int g;
    //cin>>g;
  }
  double cost=0;
  for(int x=0; x<n; x++)
    cost+=at(x,assignedx(x));
  return cost;
}



bool AssignmentMatrix::addx(const int x){
  if(foundx(x)) return false;
  foundx(x)=1;
  for(int y=0; y<n; y++)
    if(at(x,y)>yx(x)+yy(y)-0.000001 && assignedx(x)!=y && !foundy(y)){
      if(assignedy(y)==-1){ // found new alternating path!!!
	foundy(y)=1;
	assignedy(y)=x;
	assignedx(x)=y;
	//cout<<"added "<<x<<"--"<<y<<endl;
	return true;
      }else{
	foundy(y)=1;
	if(addx(assignedy(y))){
	  assignedy(y)=x;
	  assignedx(x)=y;
	  return true;
	}
      }
    }
  return false;
}



bool AssignmentMatrix::Augment(){
  foundx.fill(0);
  foundy.fill(0);
  bool success=0;
  for(int x=0; x<n; x++)
    if(assignedx(x)==-1 && addx(x)){
      success=1;
      Nassignments++;
      break;
    }
  return success;
}
