#include "BlockMatrix.hpp"

template<class Matrix>
static void fillBlockMatrix(
  Matrix &B,
  typename std::vector<Matrix *>::const_iterator begin,
  typename std::vector<Matrix *>::const_iterator end)
{
  int row_offset = 0, col_offset = 0;

  typename std::vector<Matrix *>::const_iterator k;
  for(k = begin; k < end; k++) {
    const Matrix &block = **k;

    for(int i = 0; i < block.Nrows; i++) {
      for(int j = 0; j < block.Ncols; j++) {
	B(row_offset + i, col_offset + j) = block(i, j);
      }
    }

    row_offset += block.Nrows;
    col_offset += block.Ncols;
  }
}


template<class Matrix>
static void fillBlockMatrix(
  Matrix &B,
  typename std::vector<const Matrix *>::const_iterator begin,
  typename std::vector<const Matrix *>::const_iterator end)
{
  int row_offset = 0, col_offset = 0;

  typename std::vector<const Matrix *>::const_iterator k;
  for(k = begin; k < end; k++) {
    const Matrix &block = **k;

    for(int i = 0; i < block.Nrows; i++) {
      for(int j = 0; j < block.Ncols; j++) {
	B(row_offset + i, col_offset + j) = block(i, j);
      }
    }

    row_offset += block.Nrows;
    col_offset += block.Ncols;
  }
}

template<class Matrix>
static int totalRows(const std::vector<Matrix *> &blocks)
{
  int rows = 0;
  for(int k = 0; k < blocks.size(); k++) {
    rows += (*blocks[k]).Nrows;
  }
  return rows;
}

template<class Matrix>
static int totalCols(const std::vector<Matrix *> &blocks)
{
  int cols = 0;
  for(int k = 0; k < blocks.size(); k++) {
    cols += (*blocks[k]).Ncols;
  }
  return cols;
}

template<class Matrix>
static int totalRows(const std::vector<const Matrix *> &blocks)
{
  int rows = 0;
  for(int k = 0; k < blocks.size(); k++) {
    rows += (*blocks[k]).Nrows;
  }
  return rows;
}

template<class Matrix>
static int totalCols(const std::vector<const Matrix *> &blocks)
{
  int cols = 0;
  for(int k = 0; k < blocks.size(); k++) {
    cols += (*blocks[k]).Ncols;
  }
  return cols;
}

Fmatrix blockMatrix(const std::vector<Fmatrix *> &blocks)
{
  Fmatrix B(totalRows(blocks), totalCols(blocks), 0);
  fillBlockMatrix(B, blocks.begin(), blocks.end());
  return B;
}

Fmatrix blockMatrix(const std::vector<const Fmatrix *> &blocks)
{
  Fmatrix B(totalRows(blocks), totalCols(blocks), 0);
  fillBlockMatrix(B, blocks.begin(), blocks.end());
  return B;
}

FGmatrix blockMatrix(const std::vector<FGmatrix *> &blocks)
{
  FGmatrix B(false, totalRows(blocks), totalCols(blocks), 0);
  fillBlockMatrix(B, blocks.begin(), blocks.end());
  return B;
}

FGmatrix blockMatrix(const std::vector<const FGmatrix *> &blocks)
{
  FGmatrix B(false, totalRows(blocks), totalCols(blocks), 0);
  fillBlockMatrix(B, blocks.begin(), blocks.end());
  return B;
}
