#include "FGmatrix.hpp"

//#define _NO_GSL

#include <cmath>
#include <fstream>
#ifndef _NO_GSL
#include <gsl/gsl_linalg.h>
#endif
#include <iostream>
#include <sstream>
#include <cstdlib>
#include <cstdio>
#include <cassert>

#include <Eigen/SVD>
#include <Eigen/StdVector>
#include <Eigen/Dense>
#include <Eigen/LU>
#include <Eigen/QR>
#include <Eigen/Cholesky>
#include <Eigen/Eigenvalues>
#include <Eigen/Householder>
#include <Eigen/StdDeque>  
#include <Eigen/StdList>  
#include <Eigen/Core>

#define MIN(x, y) ((x) < (y) ? (x) : (y))
#define MAX(x, y) ((x) > (y) ? (x) : (y))

#include "Timer.hpp"

//////////////////////////// Basic Constructors ////////////////////////

FGmatrix::FGmatrix(const FGmatrix& o): 
  dyadic(o.dyadic), Nrows(o.Nrows), Ncols(o.Ncols){

  if(dyadic){
    a = new FIELD[Nrows];
    for(int i=0; i<Nrows; i++) {
      a[i]=o.a[i];
    }

    b = new FIELD[Ncols];
    for(int i=0; i<Ncols; i++) {
      b[i]=o.b[i];
    }
  }
  else{
    a = new FIELD[Nrows*Ncols];
    for(int i=0; i<Nrows*Ncols; i++) {
      a[i] = o.a[i];
    }

    b = NULL;
  }
}

FGmatrix::FGmatrix(const bool _dyadic, const int _Nrows, const int _Ncols): 
  dyadic(_dyadic), Nrows(_Nrows), Ncols(_Ncols){

  if(dyadic) {
    a = new FIELD[Nrows];
    b = new FIELD[Ncols];
  }
  else {
    a = new FIELD[Nrows*Ncols];
    b = NULL;
  }
}

FGmatrix::FGmatrix(const bool _dyadic, const int _Nrows, const int _Ncols, const FIELD val): 
  dyadic(_dyadic), Nrows(_Nrows), Ncols(_Ncols){

  if(dyadic) {
    a = new FIELD[Nrows];
    for(int i =0; i < Nrows; i++) {
      a[i] = val;
    }
 
    b = new FIELD[Ncols];
    for(int i = 0; i < Ncols; i++) {
      b[i] = 1;
    }
  }
  else {
    a = new FIELD[Nrows*Ncols];
    for(int i = 0; i < Nrows*Ncols; i++) {
      a[i] = val;
    }

    b = NULL;
  }
}

FGmatrix::FGmatrix(const bool _dyadic, const int _Nrows, const int _Ncols, const Identity& dummy):
  dyadic(_dyadic), Nrows(_Nrows), Ncols(_Ncols)
{
  if(dyadic){
    a = new FIELD[Nrows];
    b = new FIELD[Ncols];
  }
  else {
    a = new FIELD[Nrows*Ncols]; 
    for(int i = 0; i < Nrows; i++) { 
      for(int j = 0; j < Ncols; j++) {
	at(i,j) = (i == j ? 1.0 : 0.0);
      }
    }

    b = NULL;
  }
}

////////////////////////////// Assignment Operators //////////////////////////

const FGmatrix& FGmatrix::operator = (const FGmatrix& o)
{
  // Cleanup old
  delete a;
  if(dyadic) {
    delete b;
  }

  // Set new properties
  dyadic = o.dyadic;
  Nrows = o.Nrows;
  Ncols = o.Ncols;

  // Alllocate + copy data
  if(dyadic) {
    a = new FIELD[Nrows];
    for(int i = 0; i < Nrows; i++) {
      a[i] = o.a[i];
    }

    b = new FIELD[Ncols];
    for(int i = 0; i < Ncols; i++) {
      b[i] = o.b[i];
    }
  }
  else{
    a = new FIELD[Nrows*Ncols];
    for(int i = 0; i < Nrows*Ncols; i++) {
      a[i] = o.a[i];
    }
  }

  return *this;
}

const FGmatrix& FGmatrix::operator += (const FGmatrix& o)
{
  assert(!dyadic);

  if(o.dyadic) {
    for(int i = 0; i < Nrows; i++) { 
      for(int j = 0; j < Ncols; j++) {
	at(i,j) += o(i,j);
      }
    }
  }
  else {
    for(int i = 0; i < Nrows*Ncols; i++) {
      a[i] += o.a[i];
    }
  }

  return *this;
}

const FGmatrix& FGmatrix::operator -= (const FGmatrix& o){
  assert(!dyadic);

  for(int i = 0; i < Nrows*Ncols; i++) {
    a[i] -= o.a[i];
  }

  return *this;
}

const FGmatrix& FGmatrix::operator *= (const FIELD o){
  if(dyadic) {
    for(int i = 0; i < Nrows; i++) {
      a[i] *= o;
    }
  }
  else {
    for(int i = 0; i < Nrows*Ncols; i++) {
      a[i] *= o;
    }
  }

  return *this;
}

///////////////////////////// Expression Constructors ////////////////////////

// FGmatrix::FGmatrix(const Sum<FGmatrix,FGmatrix>& args):
//   dyadic(args.left.dyadic), Nrows(args.left.Nrows), Ncols(args.left.Ncols){

//   assert(!dyadic);

//   a = new FIELD[Nrows*Ncols];
//   for(int i=0; i<Nrows*Ncols; i++)
//     a[i] = args.left.a[i] + args.right.a[i];

//   b = NULL;
// }


//DP change 04142012
FGmatrix::FGmatrix(const Sum<FGmatrix,FGmatrix>& args):
  dyadic(0), Nrows(args.left.Nrows), Ncols(args.right.Ncols){
  //dyadic made 0 for all matrices-DP change
  
  //assert(!dyadic);
 

  const FGmatrix& left=args.left;
  const FGmatrix& right=args.right;
 
  if(left.dyadic || right.dyadic){
   
    
    a=new FIELD[Nrows*Ncols];
 
    FIELD t=0; for(int i=0; i<left.Ncols; i++){
      for(int j=0; j<left.Nrows; j++){
	t = left.a[i]*left.b[j]+right.a[i]*right.b[j];
	a[i*Ncols+j]=t;
      }
    }
    b = NULL;
    return;}
  else{
    a=new FIELD[Nrows*Ncols];
    for(int i=0; i<Nrows*Ncols; i++){
      a[i] = args.left.a[i] + args.right.a[i];
    }
    b = NULL;
  }
}

FGmatrix::FGmatrix(const Product<FGmatrix,FGmatrix>& args):
  dyadic(args.left.dyadic || args.right.dyadic),
  Nrows(args.left.Nrows),
  Ncols(args.right.Ncols) {

  const FGmatrix& left=args.left;
  const FGmatrix& right=args.right;
  if(dyadic){
    a=new FIELD[Nrows]; b=new FIELD[Ncols];
    if(left.dyadic && right.dyadic){
      FIELD t=0; for(int i=0; i<left.Ncols; i++) t+=left.b[i]*right.a[i];
      for(int i=0; i<Nrows; i++) a[i]=left.a[i]*t;
      for(int i=0; i<Ncols; i++) b[i]=right.b[i];
      return;}
    if(left.dyadic){
      for(int i=0; i<Nrows; i++) a[i]=left.a[i];
      for(int j=0; j<Ncols; j++){FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left.b[k]*right(k,j); b[j]=t;}
      return;}
    if(right.dyadic){
      for(int i=0; i<Nrows; i++){FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left(i,k)*right.a[k]; a[i]=t;}
      for(int j=0; j<Nrows; j++) b[j]=right.b[j];
      return;}
  }else{
    a=new FIELD[Nrows*Ncols];
    for(int i=0; i<Nrows; i++) for(int j=0; j<Ncols; j++)
      {FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left(i,k)*right(k,j); a[i*Ncols+j]=t;}
    b = NULL;
  }
}


/**
 * @warning: single column/rows matrices are only partially supported at this point!!
 */
FGmatrix::FGmatrix(const Product<FGmatrix,Transpose<FGmatrix> >& args):
  dyadic(args.left.dyadic||args.right.val.dyadic||args.left.Ncols==1 ||args.right.val.Ncols==1),
  Nrows(args.left.Nrows),Ncols(args.right.val.Nrows){
  if(Nrows==1 || Ncols==1) dyadic=0;
  const FGmatrix& left=args.left;
  const FGmatrix& right=args.right.val;
  if(dyadic){
    a=new FIELD[Nrows]; b=new FIELD[Ncols];
    if(left.dyadic && right.dyadic){
      FIELD t=0; for(int i=0; i<left.Ncols; i++) t+=left.b[i]*right.b[i];
      for(int i=0; i<Nrows; i++) a[i]=left.a[i]*t;
      for(int i=0; i<Ncols; i++) b[i]=right.a[i];
      return;}
    if(left.dyadic){
      for(int i=0; i<Nrows; i++) a[i]=left.a[i];
      for(int j=0; j<Ncols; j++){FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left.b[k]*right(j,k); b[j]=t;}
      return;}
    if(right.dyadic){
      for(int i=0; i<Nrows; i++){FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left(i,k)*right.b[k]; a[i]=t;}
      for(int j=0; j<Nrows; j++) b[j]=right.a[j];
      return;}
    if(left.Ncols==1) for(int i=0; i<Nrows; i++) a[i]=left.a[i]; else for(int i=0; i<Nrows; i++) a[i]=left(i,0);
    if(right.Ncols==1) for(int i=0; i<Ncols; i++) b[i]=right.a[i]; else for(int i=0; i<Ncols; i++) b[i]=right(i,0);
  }else{
    a=new FIELD[Nrows*Ncols];
    for(int i=0; i<Nrows; i++) for(int j=0; j<Ncols; j++)
      {FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left(i,k)*right(j,k); a[i*Ncols+j]=t;}
    b = NULL;
  }
}


FGmatrix::FGmatrix(const Product<FGmatrix, Inverse<FGmatrix> > &args) :
  a(NULL), b(NULL)
{
  const FGmatrix &left = args.left;
  const FGmatrix &right = args.right.val;

  //FGmatrix rightinv = right.pseudoInverse();
  //(*this) = FGmatrix(Product<FGmatrix, FGmatrix>(left, rightinv));

  FGmatrix leftT = left.transpose();
  FGmatrix rightT = right.transpose();
  FGmatrix thisT(Product<Inverse<FGmatrix>, FGmatrix>(rightT, leftT));

  //FGmatrix thisQR = thisT.transpose();
  *this = thisT.transpose();

  //cout<<"Error = "<<(thisQR - (*this)).norm2()<<endl;
}


FGmatrix::FGmatrix(const Product<Inverse<FGmatrix>, FGmatrix> & args) :
  a(NULL), b(NULL)
{
  const FGmatrix &left = args.left.val;
  const FGmatrix &right = args.right;

  *this = left.solveQR(right, 1e-6);
}


FGmatrix::FGmatrix(const Product<Transpose<FGmatrix>,FGmatrix>& args):
  dyadic(args.left.val.dyadic||args.right.dyadic),Nrows(args.left.val.Ncols),Ncols(args.right.Ncols){
  const FGmatrix& left=args.left.val;
  const FGmatrix& right=args.right;
  if(dyadic){
    a=new FIELD[Nrows]; b=new FIELD[Ncols];
    if(left.dyadic && right.dyadic){
      FIELD t=0; for(int i=0; i<left.Ncols; i++) t+=left.a[i]*right.a[i];
      for(int i=0; i<Nrows; i++) a[i]=left.b[i]*t;
      for(int i=0; i<Ncols; i++) b[i]=right.b[i];
      return;}
    if(left.dyadic){
      for(int i=0; i<Nrows; i++) a[i]=left.b[i];
      for(int j=0; j<Ncols; j++){FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left.a[k]*right(k,j); b[j]=t;}
      return;}
    if(right.dyadic){
      for(int i=0; i<Nrows; i++){FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left(k,i)*right.a[k]; a[i]=t;}
      for(int j=0; j<Nrows; j++) b[j]=right.b[j];
      return;}
  }else{
    a=new FIELD[Nrows*Ncols];
    for(int i=0; i<Nrows; i++) for(int j=0; j<Ncols; j++)
      {FIELD t=0; for(int k=0; k<left.Nrows; k++) t+=left(k,i)*right(k,j); a[i*Ncols+j]=t;}
    b = NULL;
  }
}


FGmatrix::FGmatrix(const Product<Transpose<FGmatrix>,Transpose<FGmatrix> >& args):
  dyadic(args.left.val.dyadic||args.right.val.dyadic),Nrows(args.left.val.Ncols),Ncols(args.right.val.Ncols){
  const FGmatrix& left=args.left.val;
  const FGmatrix& right=args.right.val;
  if(dyadic){
    a=new FIELD[Nrows]; b=new FIELD[Ncols];
    if(left.dyadic && right.dyadic){
      FIELD t=0; for(int i=0; i<left.Ncols; i++) t+=left.a[i]*right.b[i];
      for(int i=0; i<Nrows; i++) a[i]=left.b[i]*t;
      for(int i=0; i<Ncols; i++) b[i]=right.a[i];
      return;}
    if(left.dyadic){
      for(int i=0; i<Nrows; i++) a[i]=left.b[i];
      for(int j=0; j<Ncols; j++){FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left.a[k]*right(j,k); b[j]=t;}
      return;}
    if(right.dyadic){
      for(int i=0; i<Nrows; i++){FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left(k,i)*right.b[k]; a[i]=t;}
      for(int j=0; j<Nrows; j++) b[j]=right.a[j];
      return;}
  }else{
    a=new FIELD[Nrows*Ncols];
    for(int i=0; i<Nrows; i++) for(int j=0; j<Ncols; j++)
      {FIELD t=0; for(int k=0; k<left.Ncols; k++) t+=left(k,i)*right(j,k); a[i*Ncols+j]=t;}
    b = NULL;
  }
}



FGmatrix::FGmatrix(const Product<FGmatrix,FIELD>& args):
  dyadic(args.left.dyadic),Nrows(args.left.Nrows),Ncols(args.left.Ncols){
  const FGmatrix& left=args.left;
  const FIELD right=args.right;
  if(dyadic){
    a=new FIELD[Nrows]; for(int i=0; i<Nrows; i++) a[i]=left.a[i]*right;
    b=new FIELD[Ncols]; for(int i=0; i<Ncols; i++) b[i]=left.b[i];}
  else{
    a=new FIELD[Nrows*Ncols];
    for(int i=0; i<Nrows*Ncols; i++) a[i]=left.a[i]*right;
    b = NULL;
  }
}

FGmatrix::FGmatrix(const HadamardProduct<FGmatrix, FGmatrix> &args) :
  dyadic(false),
  Nrows(args.left.Nrows),
  Ncols(args.left.Ncols)
{
  assert(args.left.Nrows == args.right.Nrows);
  assert(args.left.Ncols == args.right.Ncols);
  const FGmatrix& left=args.left;
  const FGmatrix& right=args.right;
  a = new FIELD[Nrows*Ncols];
  for(int i = 0; i < Nrows; i++) {
    for(int j = 0; j < Ncols; j++) {
      (*this)(i, j) = args.left(i,j) * args.right(i, j);
    }
  }

  b = NULL;
}

FGmatrix::FGmatrix(const HadamardDivision<FGmatrix, FGmatrix> &args) :
  dyadic(false),
  Nrows(args.left.Nrows),
  Ncols(args.left.Ncols)
{
  // cout<<"args.left.Nrows "<<args.left.Nrows<<" args.right.Nrows "<<args.right.Nrows<<endl;
//   getchar();
  assert(args.left.Nrows == args.right.Nrows);
  assert(args.left.Ncols == args.right.Ncols);

  const FGmatrix& left=args.left;
  const FGmatrix& right=args.right;

  a = new FIELD[Nrows*Ncols];
  for(int i = 0; i < Nrows; i++) {
    for(int j = 0; j < Ncols; j++) {
      if(fabs(right(i, j)) > 1e-6){
	(*this)(i, j) = args.left(i,j) / args.right(i, j);
      }
    }
  }
  b = NULL;
}

/////////////////////////////// Methods //////////////////////////

FIELD FGmatrix::norm2() const
{
  if(dyadic){
    FIELD t1 = 0;
    for(int i = 0; i < Nrows; i++) {
      t1 += a[i]*a[i];
    }

    FIELD t2 = 0;
    for(int i = 0; i < Ncols; i++) {
      t2 += b[i] * b[i];
    }

    return sqrt(t1*t2);
  }
  else{
    FIELD t = 0;
    for(int i = 0; i < Nrows*Ncols; i++) {
      t += a[i]*a[i];
    }
    return sqrt(t);
  }
}

FIELD FGmatrix::trace() const
{
  FIELD t = 0;

  for(int i = 0; i < Nrows; i++) {
    t += at(i,i);
  }

  return t;
}

bool FGmatrix::isNaN() const
{
  if(dyadic) {
    for(int i = 0; i < Nrows; i++) {
      if(a[i] != a[i]) {
	return true;
      }
    }

    for(int j = 0; j < Ncols; j++) {
      if(b[j] != b[j]) {
	return true;
      }
    }
  }
  else {
    for(int i = 0; i < Nrows; i++) {
      for(int j = 0; j < Ncols; j++) {
	if((*this)(i, j) != (*this)(i, j)) {
	  return true;
	}
      }
    }
  }

  return false;
}

FGmatrix FGmatrix::transpose() const
{
  FGmatrix T(dyadic, Ncols, Nrows);

  if(dyadic) {
    for(int i = 0; i < Ncols; i++) {
      T.a[i] = b[i];
    }
    for(int i = 0; i < Nrows; i++) {
      T.b[i] = a[i];
    }
  }
  else {
    for(int i = 0; i < Nrows; i++) {
      for(int j = 0; j < Ncols; j++) {
	T(j, i) = (*this)(i, j);
      }
    }
  }

  return T;
}


FIELD FGmatrix::inp(const FGmatrix& o) const {
  if(dyadic){
    FIELD t1=0; for(int i=0; i<Nrows; i++) t1+=a[i]*o.a[i];
    FIELD t2=0; for(int i=0; i<Ncols; i++) t2+=b[i]*o.b[i];
    return t1*t2;}
  else{ FIELD t=0; for(int i=0; i<Nrows*Ncols; i++) t+=a[i]*o.a[i]; return t;}
}



string FGmatrix::str() const{
  ostringstream result; 
  result.precision(STR_PRECISION);
  //if(dyadic) cout<<"Dyadic!"<<endl;
  for(int i=0; i<Nrows; i++){
    for(int j=0; j<Ncols; j++){
      FIELD e; if(dyadic) e=a[i]*b[j]; else e=a[i*Ncols+j];
      if(abs(e)>0.00001) result<<e<<" "; else result<<"0"<<" ";
    }
    result<<"\n";}
  return result.str();
}


ostream& operator << (ostream& stream, const FGmatrix& o){
  //stream.precision(STR_PRECISION);

  for(int i=0; i<o.Nrows; i++){
    for(int j=0; j<o.Ncols; j++){
      FIELD e = o(i, j);

      if(abs(e) > 0.00001) {
	stream<<e<<" "; 
      }
      else {
	stream<<"0"<<" ";
      }

      //stream<<o(i,j)<<' ';
    }
    stream<<'\n';
  }

  return stream;
}



FGmatrix::Random::Random(const bool _dyadic, const int _Nrows, const int _Ncols): 
  FGmatrix(_dyadic,_Nrows,_Ncols){
  if(dyadic){
    for(int i=0; i<Nrows; i++) a[i]=((double) rand())/((double)RAND_MAX+1);
    for(int i=0; i<Ncols; i++) b[i]=((double) rand())/((double)RAND_MAX+1);
  }
  else for(int i=0; i<Nrows*Ncols; i++) a[i]=((double) rand())/((double)RAND_MAX+1);  
}


FGmatrix FGmatrix::pseudoInverse(double tol) const
{
  //
  Eigen::MatrixXd inputMat(Nrows,Ncols);
  
  for(int i = 0; i < Nrows; i++) {
    for(int j = 0; j < Ncols; j++) {
      inputMat(i,j) = (*this)(i, j);
    }	
  }

  // Calculate SVD, split into U,Sigma,V matrices
  Timer timer;
  cout<<"Computing "<<Nrows<<'x'<<Ncols<<" SVD... "; cout.flush();
  Eigen::JacobiSVD< Eigen::MatrixXd > svd(inputMat);
  svd.compute(inputMat, Eigen::ComputeThinU | Eigen::ComputeThinV);
  cout<<"done in "<<Timer::timeString(timer.elapsed())<<endl;

  const Eigen::MatrixXd &matU = svd.matrixU();
  const Eigen::MatrixXd &matV = svd.matrixV();

  Eigen::MatrixXd sigmamat = Eigen::MatrixXd::Zero(matU.cols(), matV.rows());
  sigmamat.diagonal() = svd.singularValues();

  
  // Check SVD against original matrix
  Eigen::MatrixXd avalidation = matU * sigmamat * matV.transpose();
  assert(avalidation.rows() == Nrows && avalidation.cols() == Ncols);
  
  if((inputMat).isApprox(avalidation, 1e-12)) {
    //cout<<"SVD is quite accurate"<<endl;
  }
  else {
    cerr<<"SVD not accurate"<<endl;
  }

  // Calculate pseudoinverse
  Eigen::MatrixXd sigmainvrounded = Eigen::MatrixXd::Zero(sigmamat.rows(), sigmamat.cols());
  for(int i = 0; i < MIN(sigmamat.rows(), sigmamat.cols()); i++) {
    if(fabs(sigmamat(i,i)) > tol) {
      sigmainvrounded(i,i) = 1.0 / sigmamat(i,i);
    }
  }

  Eigen::MatrixXd outputMat = matV * sigmainvrounded.transpose() * matU.transpose();
  assert(outputMat.rows() == Ncols && outputMat.cols() == Nrows);

  // Return result
  FGmatrix inv(false, Ncols, Nrows);

  for(int i = 0; i < Ncols; i++)  {
    for(int j = 0; j < Nrows; j++) {
      inv(i, j) = outputMat(i, j);
    }
  }

  return inv;
}


/////////////////////////////// Input/Output //////////////////////////

FGmatrix::FGmatrix(ifstream& ifs, const int _Nrows, const int _Ncols, const bool binary): 
  dyadic(0), Nrows(_Nrows), Ncols(_Ncols){
  a=new FIELD[Nrows*Ncols];
  if(binary){ifs.read((char*)a,sizeof(FIELD)*Nrows*Ncols); return;}
  for(int i=0; i<Nrows*Ncols; i++) ifs>>a[i];
}


FGmatrix::FGmatrix(const string filename): dyadic(0){
  // cout<<"Loading "<<filename<<" ..."<<endl;
  ifstream ifs(filename.c_str());
  Nrows=0; Ncols=0;
  FIELD b;
  while(ifs.peek()!='\n'){
    ifs>>b; Ncols++;
  }
  while(ifs.good()){
    for(int i=0; i<Ncols;i++) ifs>>b;
    Nrows++;
  }
  a=new FIELD[Nrows*Ncols];
  ifs.close(); 
  ifstream ifs2(filename.c_str());
  for(int i=0; i<Nrows*Ncols; i++){
    ifs2>>a[i];
    // if(i%10000==0){cout<<"*"; cout.flush();}
  }
  // cout<<endl;
  ifs2.close();
}

int FGmatrix::writeto(ofstream& ofs, const bool binary){
  // does not work for dyadic
  if(binary){ofs.write((char*)a,sizeof(double)*Nrows*Ncols); return 0;}
}

///////////////////////////////// Binary Operators ////////////////////

FGmatrix operator + (const FGmatrix &A, const FGmatrix &B)
{
  FGmatrix sum(A);
  sum += B;
  return sum;
}

FGmatrix operator - (const FGmatrix &minuend, const FGmatrix &subtrahend)
{
  FGmatrix difference(minuend);
  difference -= subtrahend;
  return difference;
}
