//#define _NO_GSL

#include <cstdlib>
#include <sysexits.h>
#include <cmath>
#include <fstream>
#include <sstream>
#ifndef _NO_GSL
#include <gsl/gsl_linalg.h>
#endif

#include "Fmatrix.hpp"


Fmatrix::Fmatrix(const Fmatrix& o): Nrows(o.Nrows), Ncols(o.Ncols){
  a=new FIELD[Nrows*Ncols];
  for(int i=0; i<Nrows*Ncols; i++) a[i]=o.a[i];
}



Fmatrix::Fmatrix(const int _Nrows, const int _Ncols): Nrows(_Nrows), Ncols(_Ncols){
  a=new FIELD[Nrows*Ncols];
//cout<<"fmatrix "<<Nrows<<endl;
}



Fmatrix::Fmatrix(const int _Nrows, const int _Ncols, const FIELD val): Nrows(_Nrows), Ncols(_Ncols){
  a=new FIELD[Nrows*Ncols];
  for(int i=0; i<Nrows*Ncols; i++) a[i]=val;
}



Fmatrix::Fmatrix(const int _Nrows, const int _Ncols, const Identity& dummy): Nrows(_Nrows), Ncols(_Ncols){
  a=new FIELD[Nrows*Ncols];
  for(int i=0; i<Nrows; i++) 
    for(int j=0; j<Ncols; j++)
      at(i,j)=(i==j);
}



Fmatrix::Fmatrix(const Fmatrix& A, const Fmatrix& B): Nrows(A.Ncols),Ncols(B.Ncols){
  a=new FIELD[Nrows*Ncols];
  for(int i=0; i<Nrows; i++)
    for(int j=0; j<Ncols; j++){
      FIELD t=0;
      for(int k=0; k<A.Nrows; k++)
	t+=A(k,i)*B(k,j);
      at(i,j)=t;
    }
}



Fmatrix::Fmatrix(const string filename){
  ifstream ifs(filename.c_str());
  if(ifs.fail()) {
    std::cerr<<"Failed to open matrix file \""<<filename<<'"'<<endl;
    exit(EX_NOINPUT);
  }

  Nrows=0; Ncols=0;
  FIELD b;

  while(ifs.peek() != '\n'){
    ifs>>b;
    while(isspace(ifs.peek()) && ifs.peek() != '\n') {
      ifs.get();
    }

    Ncols++;
  }
  while(ifs.good()){
    for(int i=0; i<Ncols;i++) ifs>>b;
    Nrows++;
  }
  a=new FIELD[Nrows*Ncols];
  ifs.close(); 
  ifstream ifs2(filename.c_str());
  for(int i=0; i<Nrows*Ncols; i++){
    ifs2>>a[i];
    // if(i%10000==0){cout<<"*"; cout.flush();}
  }
  // cout<<endl;
  ifs2.close();
}


Fmatrix::Fmatrix(ifstream& ifs, const int _Nrows, const int _Ncols): Nrows(_Nrows), Ncols(_Ncols){
  a=new FIELD[Nrows*Ncols];
  for(int i=0; i<Nrows*Ncols; i++) ifs>>a[i];
}

const Fmatrix &Fmatrix::operator = (const Fmatrix &assign)
{
  if(&assign == this) {
    return *this;
  }

  delete [] a;
  Nrows = assign.Nrows;
  Ncols = assign.Ncols;
  a = new FIELD[Nrows * Ncols];
  for(int i = 0; i < Nrows * Ncols; i++) {
    a[i] = assign.a[i];
  }

  return *this;
}


Fmatrix::~Fmatrix(){
  delete [] a;
}



Fmatrix& Fmatrix::operator+=(Fmatrix& o){
  for(int i=0; i<Nrows*Ncols; i++) 
    a[i]+=o.a[i];
  return *this;
}


Fmatrix& Fmatrix::operator*=(FIELD o){
  for(int i=0; i<Nrows*Ncols; i++) 
    a[i]*=o;
  return *this;
}


Fmatrix* Fmatrix::operator*(Fmatrix& o){
  Fmatrix* result=new Fmatrix(Nrows,o.Ncols);
  for(int i=0; i<Nrows; i++)
    for(int j=0; j<o.Ncols; j++){
      FIELD t=0;
      for(int k=0; k<Ncols; k++)
	t+=at(i,k)*o.at(k,j);
      result->at(i,j)=t;
    }
  return result;
}



string Fmatrix::str() const{
  ostringstream result; 
  result.precision(STR_PRECISION);
  for(int i=0; i<Nrows; i++){
    for(int j=0; j<Ncols; j++) 
      if(abs(a[i*Ncols+j])>0.00001) 
	result<<a[i*Ncols+j]<<" ";
      else 
	result<<"0"<<" ";
      result<<"\n";}
  return result.str();
}



FIELD Fmatrix::norm2(){
  FIELD t=0;
  for(int i=0; i<Nrows*Ncols; i++)
    t+=a[i]*a[i];
  return sqrt(t);
}


double Fmatrix::tracenorm(){
  double result=0;
#ifndef _NO_GSL
  gsl_matrix* A=gsl_matrix_alloc(Nrows,Ncols);
  gsl_matrix* V=gsl_matrix_alloc(Ncols,Ncols);
  gsl_vector* S=gsl_vector_alloc(Ncols);
  gsl_vector* work=gsl_vector_alloc(Ncols);
  for(int i=0; i<Nrows*Ncols; i++)
    *(A->data+i)=a[i];
  //cout<<"svin"<<endl;
  gsl_linalg_SV_decomp(A,V,S,work);
  //cout<<"svout"<<endl;
  for(int i=0; i<Ncols; i++)
    result+=gsl_vector_get(S,i);
  gsl_matrix_free(A);
  gsl_matrix_free(V);
  gsl_vector_free(S);
  gsl_vector_free(work);
#endif
  return result;
}



FIELD Fmatrix::inp(const Fmatrix& o){
  FIELD t;
  for(int i=0; i<Nrows*Ncols; i++)
    t+=a[i]*o.a[i];
  return t;
}


Fmatrix::Random::Random(const int _Nrows, const int _Ncols):Fmatrix(_Nrows,_Ncols){
  a=new FIELD[Nrows*Ncols];
  for(int i=0; i<Nrows*Ncols; i++) a[i]=((double) rand())/((double)RAND_MAX+1);  
}


Fmatrix Fmatrix::matrixMinor(const std::vector<int> &rows, const std::vector<int> &cols) const
{
  Fmatrix M(rows.size(), cols.size());

  for(int i = 0; i < rows.size(); i++) {
    int row = rows[i];
    //cout<<"row---- "<<row<<endl;
    for(int j = 0; j < cols.size(); j++) {
      int col = cols[j];
      M(i, j) = (*this)(row, col);

    }
}
  
  return M;
}
