#ifndef _Fmatrix
#define _Fmatrix 

#include "base.h"
#include "ExpressionTemplates.hpp"

#include <string>
#include <fstream>
#include <cassert>
#include <vector>

using namespace std;

class Fmatrix{
public:

  class Random;

  Fmatrix(){};
  Fmatrix(const Fmatrix& o);
  Fmatrix(const int _Nrows, const int _Ncols);
  Fmatrix(const int _Nrows, const int _Ncols, const FIELD val);
  Fmatrix(const int _Nrows, const int _Ncols, const Identity& dummy);

  Fmatrix(const string filename);
  Fmatrix(ifstream& ifs, const int _Nrows, const int _Ncols);

  Fmatrix(const Fmatrix& A, const Fmatrix& B); // A^T \cdot B

  virtual ~Fmatrix();


  const Fmatrix &operator = (const Fmatrix &);

  FIELD& operator () (const int i, const int j) {
    assert(i < Nrows && j < Ncols);

    return a[i*Ncols + j];
  }

  FIELD operator () (const int i, const int j) const {
    assert(i < Nrows && j < Ncols);

    return a[i*Ncols + j];
  }
 
  FIELD& at(const int i, const int j){
    assert(i < Nrows && j < Ncols);

    return a[i*Ncols + j];
  }

  FIELD at(const int i, const int j) const {
    assert(i < Nrows && j < Ncols);

    return a[i*Ncols + j];
  } 

  Fmatrix& operator += (Fmatrix& o);
  Fmatrix& operator *= (FIELD o);

  Fmatrix *operator * (Fmatrix& o);
  
  FIELD norm2();
  double tracenorm();
  FIELD inp(const Fmatrix& o);
  
  string str() const;

  /**
   * Restrict the matrix to a subset of its rows + cols
   */
  Fmatrix matrixMinor(const std::vector<int> &rows, const std::vector<int> &cols) const;

  int Nrows;
  int Ncols;
  FIELD* a;

};


class Fmatrix::Random: public Fmatrix{
public:
  Random(const int _Nrows, const int _Ncols);
};

#endif
