#ifndef __SNOB2_UTILITY_INHERITABLE_VECTOR_HPP__
#define __SNOB2_UTILITY_INHERITABLE_VECTOR_HPP__

#include <vector>
#include <cassert>

/**
 * A proxy for std::vector which can safely be inherited from
 */
template<class T>
class InheritableVector
{
public:
  typedef typename std::vector<T> stl_vector;
  typedef typename stl_vector::iterator iterator;
  typedef typename stl_vector::const_iterator const_iterator;

  InheritableVector() {}
  InheritableVector(size_t n) : v(n) {}
  InheritableVector(size_t n, const T &t) : v(n, t) {}
  InheritableVector(const InheritableVector &copy) : v(copy.v) {}
  InheritableVector(const stl_vector &_v) : v(_v) {}
  
  virtual ~InheritableVector() {}

  const InheritableVector &operator = (const InheritableVector &assign)
  {
    v = assign.v;
  }

  bool empty() const {return v.empty();}
  size_t size() const {return v.size();}

  void push_back(const T &t) {v.push_back(t);}
  void pop_back() {v.pop_back();}
  void clear() {v.clear();}

  void resize(size_t n) {v.resize(n);}
  void resize(size_t n, const T &t) {v.resize(n, t);}

  T &at(size_t i) {
    assert(i < size());
    return v[i];
  }
  const T &at(size_t i) const {
    assert(i < size());
    return v[i];
  }
  T &operator [] (size_t i) {
    assert(i < size());
    return v[i];
  }
  const T &operator [] (size_t i) const {
    assert(i < size());
    return v[i];
  }

  iterator begin() {return v.begin();}
  const_iterator begin() const {return v.begin();}
  iterator end() {return v.end();}
  const_iterator end() const {return v.end();}

  operator std::vector<T> () {return v;}

private:
  std::vector<T> v;
};

#endif // #ifndef __SNOB2_UTILITY_INHERITABLE_VECTOR_HPP__
