#ifndef __UTILITY_FGMATRIX_LAPACK_HPP__
#define __UTILITY_FGMATRIX_LAPACK_HPP__

#include "FGmatrix.hpp"

extern "C" {
#include <f2c.h>
}
#include <cassert>

/**
 * Fill LAPACK-style column-major array with FGmatrix 
 *
 * @arg @c lda "Leading dimension" of ptr matrix, usually == A.Nrows
 */
void FGmatrixFillLAPACK(const FGmatrix &A, int lda, doublereal *ptr);


/**
 * Convert FGmatrix object to LAPACK-friendly column-major array
 *
 * @note Caller must delete [] result.
 */
doublereal *FGmatrixToLAPACK(const FGmatrix &A);


/**
 * Convert LAPACK-style column-major array to FGmatrix object
 *
 * @arg @c lda "Leading dimension" of ptr array, usually == Nrows
 */
FGmatrix FGmatrixFromLAPACK(int Nrows, int Ncols, int lda, const doublereal *ptr);

#endif // #ifndef __UTILITY_FGMATRIX_LAPACK_HPP__
