#ifndef _Matrix
#define _Matrix

#include <vector>
#include <string>
#include <sstream>
#include <fstream>
#include <math.h>

#include "base.h"
#include "Vector.hpp"

#ifdef _ops
  extern ScalarCounter ops;
#endif 

const double _EQUALITYEPSILON=0.000001;

using namespace std;


template <class TYPE>
class Matrix{
public:

  Matrix():n(1),m(1) {array=new TYPE[1]; array[0]=0;}

  Matrix(const int _n):n(_n),m(_n)
  {array=new TYPE[n*m]; for(int i=0; i<n*m; i++) if(i%(n+1)==0) array[i]=1; else array[i]=0;}

  Matrix(const int _n, const int _m):n(_n),m(_m){array=new TYPE[n*m];};

  Matrix(const int _n, const int _m, const TYPE a):n(_n),m(_m)
  {array=new TYPE[n*m]; for(int i=0; i<n*m; i++) array[i]=a;};

  Matrix(const TYPE** a, const int _n, const int _m):n(_n),m(_m){
    array=new TYPE[n*m];
    for(int i=0; i<n; i++)
      for(int j=0; j<m; j++)
	array[i*m+j]=a[i][j];
  }

  Matrix(const Matrix<TYPE>& o):n(o.n),m(o.m)
  {array=new TYPE[n*m]; for(int i=0; i<n*m; i++) array[i]=o.array[i];};

  Matrix(const int _n, const vector<Matrix<TYPE>*>& mlist);
  // Construct matrix as direct sum of square matrices 

  Matrix(const Vector<TYPE>& a, const Vector<TYPE>& b);
  // Outer product

  Matrix(const string filename);

  Matrix(ifstream& ifs, const int _n, const int _m, const bool binary=0):n(_n),m(_m){
    array=new TYPE[n*m]; ifs.read((char*)array,sizeof(TYPE)*n*m);}

 ~Matrix(){delete[] array;}

  void resize(int _n, int _m, const TYPE a){
    n=_n; m=_m;
    delete[] array;
    array=new TYPE[n*m];
    for(int i=0; i<n*m; i++) array[i]=a; 
  }

  TYPE& at(const int i, const int j){
    // if(i>n || j>>m){cout<<i<<j<<n<<m<<endl; return array[0];}
    return array[i*m+j];};

  TYPE& operator()(const int i, const int j){
    return array[i*m+j];};

  Matrix<TYPE>* operator*(const TYPE& o);
  Matrix<TYPE>* operator*(Matrix<TYPE>& o);
  Matrix<TYPE>* operator+(const Matrix<TYPE>& o);
  Matrix<TYPE>* tensorproduct(Matrix<TYPE>& o);
  Matrix<TYPE>* sandwich(Matrix<TYPE>& A, Matrix<TYPE>& B);

  Matrix<TYPE>& operator+=(const Matrix<TYPE>& o);
  Matrix<TYPE>& operator*=(const TYPE& mul);

  Matrix<TYPE>& operator=(const Matrix<TYPE>& o);

  bool operator==(const TYPE o);

  Matrix<TYPE>& fill(const TYPE& x){ for(int i=0; i<n*m; i++) array[i]=x; return *this;}
  Matrix<TYPE>& randomize(){for(int i=0; i<n*m; i++) array[i]=((TYPE) rand())/((TYPE)RAND_MAX+1); return *this;}

  // Matrix<TYPE>* tensorproduct(const Matrix<TYPE>& o) const;
  // Matrix<TYPE>* tensorproduct(const int tn, const int tm) const;


  TYPE trace() const{TYPE t=0; for(int i=0; i<n; i++) t+=array[i*(n+1)]; return t;};
  // TYPE schur2(const Matrix<TYPE>& o) const;
  TYPE norm2() const {double result=0; for(int i=0; i<n*m; i++) result+=array[i]*array[i]; return result;}
  //Matrix<TYPE>* inp(Matrix<TYPE>& o);
  TYPE inp(Matrix<TYPE>& o);

  Matrix<TYPE>* pow(const int p);


  string str() const;

  int save(const string filename);
  int writeto(ofstream& ofs, const bool binary=0);

  //private: (const removed!)
  int n;
  int m;

  TYPE* array;

};


template <class TYPE>
Matrix<TYPE>::Matrix(const int _n, const vector<Matrix<TYPE>*>& mlist):n(_n),m(_n){
  array=new TYPE[n*m];
  const int Nsummands=mlist.size();
  int total=0;
  for(int i=0; i<Nsummands; i++) total+=mlist[i]->n;
  int filled=0; 
  for(int summand=0; summand<Nsummands; summand++){
    const int size=mlist[summand]->n;
    const int offset=filled*m;
    for(int i=0; i<size; i++){
      for(int j=0; j<filled; j++) array[offset+i*m+j]=0;
      for(int j=0; j<size; j++) array[offset+i*m+filled+j]=mlist[summand]->at(i,j);
      for(int j=filled+size; j<m; j++) array[offset+i*m+j]=0;
    }
    filled+=size; 
  }
  for(int i=filled*n; i<n*m; i++) array[i]=0;
}


// This is a weird outer product flipping the numbering of i and j
template <class TYPE>
Matrix<TYPE>::Matrix(const Vector<TYPE>& a, const Vector<TYPE>& b):n(a.n),m(b.n){
  array=new TYPE[n*m];
  for(int i=0; i<n; i++)
    for(int j=0; j<m; j++)
      array[i*m+j]=a.v[n-i-1]*b.v[n-j-1];
}


template <class TYPE>
string Matrix<TYPE>::str() const{
  ostringstream result; 
  result.precision(STR_PRECISION);
  for(int i=0; i<n; i++){
    for(int j=0; j<m; j++) 
      if(fabs(array[i*m+j])>0.00001) 
	result<<array[i*m+j]<<" ";
      else 
	result<<"0"<<" ";
      result<<"\n";}
  return result.str();
}

template <class TYPE>
Matrix<TYPE>* Matrix<TYPE>::operator*(const TYPE& o){
  Matrix<TYPE>* result=new Matrix(n,m);
  for(int i=0; i<n; i++)
    for(int j=0; j<m; j++)
      result->at(i,j)=o*at(i,j);
  return result;
}

template <class TYPE>
Matrix<TYPE>* Matrix<TYPE>::operator*(Matrix<TYPE>& o){
  Matrix<TYPE>* result=new Matrix(n,o.m);
  for(int i=0; i<n; i++)
    for(int j=0; j<o.m; j++){
      TYPE t=0;
      for(int k=0; k<m; k++)
	t+=at(i,k)*o.at(k,j);
      result->at(i,j)=t;
    }
  return result;
}

template <class TYPE>
Matrix<TYPE>* Matrix<TYPE>::operator+(const Matrix<TYPE>& o){
  Matrix<TYPE>* result=new Matrix(n,m);
  for(int i=0; i<n; i++)
    for(int j=0; j<m; j++)
      result->at(i,j)=o.at(i,j)+at(i,j);
  return result;
}

template <class TYPE>
Matrix<TYPE>* Matrix<TYPE>::tensorproduct(Matrix<TYPE>& o){
  Matrix<TYPE>* result=new Matrix(n*o.n,m*o.m);
  for(int i1=0; i1<n; i1++)
    for(int i2=0; i2<o.n; i2++)
      for(int j1=0; j1<m; j1++)
	for(int j2=0; j2<o.m; j2++)
	  result->at(i1*o.n+i2,j1*o.m+j2)=o.at(i2,j2)*at(i1,j1);
  return result;
}

template <class TYPE> // Compute A' * *this * B
Matrix<TYPE>* Matrix<TYPE>::sandwich(Matrix<TYPE>& A, Matrix<TYPE>& B){
  Matrix<TYPE>* result=new Matrix(A.m,B.m);
  Matrix<TYPE> T(n,B.m);
  for(int i=0; i<n; i++)
    for(int j=0; j<B.m; j++){
      TYPE t=0;
      for(int k=0; k<m; k++)
	t+=at(i,k)*B.at(k,j);
      T(i,j)=t;
    }
  for(int i=0; i<A.m; i++)
    for(int j=0; j<B.m; j++){
      TYPE t=0;
      for(int k=0; k<n; k++)
	t+=A.at(k,i)*T.at(k,j);
      result->at(i,j)=t;
    }
  return result;
}

/*
template <class TYPE>
Matrix<TYPE>* Matrix<TYPE>::inp(Matrix<TYPE>& o){
  Matrix<TYPE>* result=new Matrix(n,o.m);
  for(int i=0; i<n; i++)
    for(int j=0; j<o.m; j++){
      TYPE t=0;
      for(int k=0; k<m; k++)
	t+=at(k,i)*o.at(k,j);
      result->at(i,j)=t;
    }
  return result;
}
*/

template <class TYPE>
TYPE Matrix<TYPE>::inp(Matrix<TYPE>& o){
  TYPE result=0;
  for(int i=0; i<n; i++)
    for(int j=0; j<m; j++)
      result+=at(i,j)*o.at(i,j);
  return result;
}


template <class TYPE>
Matrix<TYPE>& Matrix<TYPE>::operator+=(const Matrix<TYPE>& o){
  for(int i=0; i<n*m; i++) array[i]+=o.array[i];
#ifdef _ops
	  ops+=n*m; 
#endif
  return *this;
}

template <class TYPE>
Matrix<TYPE>& Matrix<TYPE>::operator=(const Matrix<TYPE>& o){
  for(int i=0; i<n*m; i++) array[i]=o.array[i];
  return *this;
}

template <class TYPE>
Matrix<TYPE>& Matrix<TYPE>::operator*=(const TYPE& mul){
  for(int i=0; i<n*m; i++) array[i]*=mul;
  return *this;
}

template <class TYPE>
bool Matrix<TYPE>::operator==(const TYPE o){
  for(int i=0; i<n*m; i++) 
    if(abs(array[i]-o)>_EQUALITYEPSILON) return false;
  return true;
}

/*
template <class TYPE>
Matrix<TYPE>* Matrix<TYPE>::tensorproduct(const Matrix<TYPE>& o) const{
  Matrix<TYPE>* result=new Matrix<TYPE>(n*o.n,m*o.m);
  const int tt=m*o.m;
  for(int a=0; a<o.n; a++)
    for(int b=0; b<o.m; b++){
      const TYPE t=o.array[a*m+b];
      for(int i=0; i<n; i++)
	for(int j=0; j<m; j++)
	  result->array[(i+n*a)*tt+j+m*b]=t*array[i*m+j];
    }
  return result;
}


template <class TYPE>
Matrix<TYPE>* Matrix<TYPE>::tensorproduct(const int tn, const int tm) const{
  Matrix<TYPE>* result=new Matrix<TYPE>(n*tn,m*tm);
  const int tt=m*tm;
  for(int a=0; a<tn; a++)
    for(int b=0; b<tm; b++)
      for(int i=0; i<n; i++)
	for(int j=0; j<m; j++)
	  result->array[(i+n*a)*tt+j+m*b]=array[i*m+j];
  return result;
}



template <class TYPE>
TYPE Matrix<TYPE>::schur2(const Matrix<TYPE>& o) const{
  TYPE result;
  for(int i=0; i<n; i++) 
    for(int j=0; j<m; j++)
      result+=array[i*m+j]*o.array[j*n+i];
  return result;
}
*/



template <class TYPE>
Matrix<TYPE>* Matrix<TYPE>::pow(const int p){
  if(p<2) return new Matrix<TYPE>(*this);
  Matrix<TYPE>* result=(*this)*(*this);
  for(int i=2; i<p; i++){
    Matrix<TYPE>* t=(*this)*(*result);
    delete result;
    result=t;
  }
  return result;
}

template <class TYPE>
Matrix<TYPE>::Matrix(const string filename){
  cout<<"Loading "<<filename<<" ..."<<endl;
  ifstream ifs(filename.c_str());
  n=0;m=0;
  TYPE b;
  while(ifs.peek()!='\n'){
    ifs>>b; m++;
  }
  while(ifs.good()){
    for(int i=0; i<m;i++) ifs>>b;
    n++;
  }
  array=new TYPE[n*m];
  ifs.close(); 
  ifstream ifs2(filename.c_str());
  for(int i=0; i<n*m; i++){
    ifs2>>array[i];
    if(i%10000==0){cout<<"*"; cout.flush();}
  }
  cout<<endl;
  ifs2.close();
}


template<class TYPE>
int Matrix<TYPE>::save(const string filename){
  cout<<"Saving "<<filename<<" ..."<<endl;
  ofstream ofs(filename.c_str());
  for(int i=0; i<n; i++){
    for(int j=0; j<m; j++) 
      ofs<<" "<<array[i*m+j];
    ofs<<"\n";
  }
  ofs.close();
}


template<class TYPE>
int Matrix<TYPE>::writeto(ofstream& ofs, const bool binary){
  if(binary){ofs.write((char*)array, sizeof(TYPE)*n*m); return 0;}
}



#endif
