#include "Permutation.hpp"

#include <stdarg.h>
#include <iostream>
#include <stdlib.h>
#include <cassert>


Permutation::Permutation(const pair<int,int>& p){
  n=p.first;
  int m=p.second-1;
  int mult=1;
  push_back(1);
  for(int k=2; k<=n; k++){
    int r=m%k;
    m=(m-r)/k;
    r=k-r; // changed 7/15/09
    for(int i=0; i<size(); i++) if(at(i)>=r) at(i)++;
    push_back(r);
  }
}



Permutation::Permutation(int a1, int a2, ...){
  va_list params;
  int arg=a2;
  push_back(a1);
  va_start(params,a2);
  while(arg!=0){
    push_back(arg);
    arg=va_arg(params, int);
  }
  va_end(params);
  n=size();
}



Permutation::Permutation(const CClist& cclist){
  for(int i=0; i<cclist.vec.size(); i++){
    int r=cclist.vec[i];
    for(int j=0; j<size(); j++) if(at(j)>=r) at(j)++;
    push_back(r);
  }
  n = size();
}



ostream& operator<<(ostream& stream, const Permutation& o){
  stream<<"[ "; for(int i=0; i<o.size(); i++) stream<<o[i]<<" "; stream<<"]";
  return stream;
}



Permutation::Random::Random(const int _n){
  n=_n;
  for(int i=1; i<=n; i++){
    int r=(int)(((double) rand())/((double)RAND_MAX+1)*i+1);
    for(int j=0; j<size(); j++) if (at(j)>=r) at(j)++;
    push_back(r);
  }
}



double factorial(const int n){
  double res=1; for(int i=2; i<=n; i++) res*=i; 
  return res;
}


std::vector<int> Permutation::cclist() const
{
  std::vector<int> vec(n);

  for(int i = n; i >= 1; i--) {
    int r = at(i-1);

    for(int j = n; j >= i+1; j--) {
      if(vec[j-1] < r) {
	r--;
      }
    }

    vec[i-1] = r;
  }

  return vec;
}

vector<Permutation> Permutation::enumerate(int n)
{
  vector<Permutation> permsn;

  // Base cases
  if(n == 0) {
    return permsn;
  }
  else if(n == 1) {
    permsn.push_back(Permutation(1, 0));
    return permsn;
  }

  // Recurse
  vector<Permutation> permsn1 = enumerate(n-1);

  for(int i = 0; i < permsn1.size(); i++) {
    for(int j = n-1; j >= 0; j--) {

      // Take lower-level permutation and swap (j,n)
      Permutation p(n);

      for(int k = 0; k < n-1; k++) {
	p[k] = permsn1[i][k];
      }

      int swap = p[j];
      p[j] = n;
      p[n-1] = swap;

      // Add to list
      permsn.push_back(p);
    }
  }

  return permsn;
}

Permutation Permutation::transposition(int n, int i, int j)
{
  Permutation p(n);

  p[i-1] = j;
  p[j-1] = i;

  return p;
}

const Permutation &Permutation::compose(Permutation second)
{
  Permutation first = *this;

  assert(n == second.n);
  
  for(int i = 0; i < n; i++) {
    (*this)[i] = first[second[i]-1];
  }

  return *this;
}

Permutation Permutation::inverse() const
{
  Permutation inv(n);

  for(int i = 0; i < n; i++) {
    inv[(*this)[i]-1] = i+1;
  }

  return inv;
}

bool operator == (const Permutation &p1, const Permutation &p2)
{
  if(p1.n != p2.n) {
    return false;
  }

  for(int i = 0; i < p1.n; i++) {
    if(p1[i] != p2[i]) {
      return false;
    }
  }

  return true;
}
