#include "FGmatrix.hpp"
#include "Lapack.hpp"
#include "Timer.hpp"

#include <cmath>

extern "C" {
  int dgesdd_(char *jobz,
	      integer *m, integer *n, doublereal *A, integer *lda,
	      doublereal *S,
	      doublereal *U, integer *ldu,
	      doublereal *VT, integer *ldvt,
	      doublereal *work, integer *lwork, integer *iwork,
	      integer *info);
}


#define MIN(x, y) ((x) < (y) ? (x) : (y))
#define MAX(x, y) ((x) > (y) ? (x) : (y))



std::vector<FIELD> FGmatrix::singularValues() const
{
  int nvalues = MIN(Nrows, Ncols);

  // Input arguments
  char jobz = 'N';

  integer m = Nrows;
  integer n = Ncols;
  integer lda = m;

  doublereal *A = FGmatrixToLAPACK(*this);

  // Output arguments
  doublereal *S = new doublereal[nvalues];

  doublereal *U = NULL;
  integer ldu = 1;

  doublereal *VT = NULL;
  integer ldvt = 1;

  // Find work size
  doublereal worksize;
  integer lwork = -1, info;
  integer *iwork = new integer[8 * nvalues];

  dgesdd_(&jobz, &m, &n, A, &lda, S, U, &ldu, VT, &ldvt,
	  &worksize, &lwork, iwork,
	  &info);
  assert(info == 0);

  lwork = (integer)worksize;
  doublereal *work = new doublereal[lwork];

  // Calculate SVDs
  dgesdd_(&jobz, &m, &n, A, &lda, S, U, &ldu, VT, &ldvt,
	  work, &lwork, iwork,
	  &info);
  assert(info == 0);

  // Assemble output
  std::vector<FIELD> svec(nvalues);
  for(int i = 0; i < nvalues; i++) {
    svec[i] = (double)(S[i]);
  }
  
  // Cleanup
  delete [] work;
  delete [] iwork;
  delete [] S;
  delete [] A;

  return svec;
}


double FGmatrix::tracenorm() const
{
  if(dyadic) {
    return norm2();
  }

  std::vector<FIELD> s = singularValues();

  int n = s.size();
  double nrm = 0;
  for(int i = 0; i < n; i++) {
    nrm += s[i];
  }

  return nrm;
}

// TODO: use LAPACK
void FGmatrix::SVD(FGmatrix& rS, FGmatrix& rU, FGmatrix& rV) const { // M=U S V'
  FGmatrix* result=new FGmatrix(0,Nrows,Ncols);
  if(dyadic){
    
  }else{
#ifndef _NO_GSL
    gsl_matrix* A=gsl_matrix_alloc(Nrows,Ncols);
    gsl_matrix* V=gsl_matrix_alloc(Ncols,Ncols);
    gsl_vector* S=gsl_vector_alloc(Ncols);
    gsl_vector* work=gsl_vector_alloc(Ncols);
    for(int i=0; i<Nrows*Ncols; i++)
      *(A->data+i)=a[i];
    //coutq<<"svin"<<endl;
    gsl_linalg_SV_decomp(A,V,S,work);
    //cout<<"svout"<<endl;
    for(int i=0; i<Ncols; i++)
      rS.a[i]=gsl_vector_get(S,i);
    for(int i=0; i<Nrows; i++)
      for(int j=0; j<Ncols; j++){
	rU(i,j)=gsl_matrix_get(A,i,j);
	rV(i,j)=gsl_matrix_get(V,i,j);
      }
    gsl_matrix_free(A);
    gsl_matrix_free(V);
    gsl_vector_free(S);
    gsl_vector_free(work);
#endif
  }
}


