#include "Timer.hpp"
#include "Format.hpp"

Timer::Timer()
{
  gettimeofday(&t, NULL);
}

double Timer::elapsed() const
{
  struct timeval now;
  gettimeofday(&now, NULL);

  double s = (double)(now.tv_sec - t.tv_sec);
  s += 1e-6 * (double)(now.tv_usec - t.tv_usec);

  return s;
}

void Timer::reset()
{
  gettimeofday(&t, NULL);
}

static const char *time_units(double &s)
{
  if(s > 1e-1) {
    return "s";
  }
  else if(s >= 1e-3) {
    s *= 1e3;
    return "ms";
  }
  else if(s >= 1e-6) {
    s *= 1e6;
    return "us";
  }
  else {
    s *= 1e9;
    return "ns";
  }
}

std::string Timer::timeString(double s)
{
  const char *suffix = time_units(s);
  return stringFormat("%.2f%s", s, suffix);
}
