#ifndef _Vector
#define _Vector

#include <vector>
#include <iostream>
#include <sstream>
#include "stdarg.h"
#include <fstream>

using namespace std;

template <class TYPE>
class Vector{
public:

  Vector(const int _n):n(_n){v=new TYPE[n];}
  Vector(const int _n, const TYPE a):n(_n){v=new TYPE[n]; for(int i=0; i<n; i++) v[i]=a;}
  Vector(const Vector<TYPE>& o):n(o.n){v=new TYPE[n]; for(int i=0; i<n; i++) v[i]=o.v[i];}
  Vector(TYPE a1, TYPE a2, TYPE a3, ...);
  Vector(ifstream& ifs, const int _n, const bool binary=0);

  ~Vector(){delete[] v;}

  TYPE& operator()(const int i){return v[i];}
  TYPE& operator()(const int i) const {return v[i];}
  TYPE& operator[](const int i){return v[i];}
  TYPE& operator[](const int i) const {return v[i];}
  TYPE& at(const int i){return v[i];}
  TYPE& at(const int i) const {return v[i];}

  int size() const {return n;}

  bool operator==(const Vector<TYPE>& o){
    if(o.n!=n) return 0; for(int i=0; i<n; i++) if(v[i]!=o.v[i]) return 0; return 1;}

  TYPE operator*(Vector<TYPE>& o){TYPE t=0; for(int i=0; i<n; i++) t+=v[i]*o.v[i]; return t;}

  void fill(const TYPE val){for(int i=0; i<n; i++) v[i]=val;}

  string str(){
    ostringstream result; result<<"("; for (int i=0; i<n-1; i++) result<<v[i]<<","; result<<v[n-1]<<")"; return result.str();}

  int save(const string filename);
  int readfrom(ifstream& ifs, const bool binary=0);
  int writeto(ofstream& ofs, const bool binary=0);

  int n; // const removed!
  TYPE* v;

};



template <class TYPE>
Vector<TYPE>::Vector(TYPE a1, TYPE a2, TYPE a3, ...){
  va_list params;
  vector<TYPE> V;
  V.push_back(a1); 
  V.push_back(a2);
  TYPE p=a3;
  va_start(params,a3);
  while(p!=0){
    V.push_back(p);
    p=va_arg(params, TYPE);
  }
  va_end(params);
  n=V.size();
  v=new TYPE[n];
  for(int i=0; i<n; i++) v[i]=V[i];
}



template<class TYPE>
int Vector<TYPE>::save(const string filename){
  cout<<"Saving "<<filename<<" ..."<<endl;
  ofstream ofs(filename.c_str());
  for(int i=0; i<n; i++) ofs<<" "<<v[i];
  ofs<<"\n";
  ofs.close();
}


template<class TYPE>
Vector<TYPE>::Vector(ifstream& ifs, const int _n, const bool binary):n(_n){
  v=new TYPE[n];
  if(binary) {ifs.read(v,sizeof(TYPE)*n); return;}
}


template<class TYPE>
ostream& operator<<(ostream& stream, const Vector<TYPE>& v){
  stream<<"(";for(int i=0; i<v.n-1; i++) stream<<v[i]<<","; stream<<v[v.n-1]<<")";
  return stream;
}



template<class TYPE>
int Vector<TYPE>::writeto(ofstream& ofs, const bool binary){
  if(binary){ ofs.write((char*)v,sizeof(TYPE)*n); return 0;}
}

template<class TYPE>
int Vector<TYPE>::readfrom(ifstream& ifs, const bool binary){
  if(binary){ ifs.read((char*)v,sizeof(TYPE)*n); return 0;}
}


#endif
