#include "linalg.h"

#include <stdio.h>
#include <assert.h>
#include <f2c.h>
#include <math.h>

void linalg_print(size_t m, size_t n, const double *A)
{
  size_t i, j;

  for(i = 0; i < m; i++) {
    for(j = 0; j < n; j++) {
      printf("% 5g ", A[n*i + j]);
    }
    fputc('\n', stdout);
  }
}

extern int dgesvd_(
  const char *jobu,
  const char *jobv,
  const integer *m, const integer *n, doublereal *A, const integer *lda,
  doublereal *S,
  doublereal *U, const integer *ldu,
  doublereal *VT, const integer *ldvt,
  doublereal *work, const integer *lwork,
  integer *info);

void linalg_sqrt(size_t size, double *root, const double *mat)
{
  char jobu, jobv;
  integer m, n, lda, ldu, ldvt, lwork, info;
  doublereal worksize, *work;
  doublereal *A, *S, *U;

  size_t i, j, k;

  jobu = 'A';
  jobv = 'N';

  lda = m = (integer)size;
  n = (integer)size;

  ldu = (integer)size;
  ldvt = 1;

  A = (doublereal *)malloc((2 * size * size + size) * sizeof(doublereal));
  U = A + size*size;
  S = U + size*size;

  // Initialize input
  for(i = 0; i < size; i++) {
    for(j = 0; j < size; j++) {
      assert(mat[i*size + j] == mat[i + j*size]);

      A[size*i + j] = mat[size*i + j];
    }
  }

  // Find workspace size
  lwork = -1;
  dgesvd_(&jobu, &jobv, &m, &n, A, &lda, S, U, &ldu, NULL, &ldvt, &worksize, &lwork, &info);
  lwork = (integer)worksize;
  work = (doublereal *)malloc(lwork * sizeof(doublereal));

  // Calculate SVD
  dgesvd_(&jobu, &jobv, &m, &n, A, &lda, S, U, &ldu, NULL, &ldvt, work, &lwork, &info);

  // Calculate square root
  for(i = 0; i < size; i++) {
    S[i] = sqrt(S[i]);
  }

  for(i = 0; i < size; i++) {
    for(j = 0; j < size; j++) {
      root[i*size + j] = 0;
      for(k = 0; k < size; k++) {
        root[i*size + j] += U[i + size*k] * S[k] * U[j + size*k];
      }
    }
  }

  // Cleanup
  free(work);
  free(A);
}

extern int dger_(
  const integer *m, const integer *n, const double *alpha,
  const double *x, const integer *incx,
  const double *y, const integer *incy,
  double *A, const integer *lda);

void linalg_first_order(size_t m, size_t n, double alpha, const double *x, const double *y, double *A)
{
  integer fm, fn, inc;

  fm = (integer)m;
  fn = (integer)n;
  inc = 1;

  dger_(&fm, &fn, &alpha, x, &inc, y, &inc, A, &fm);
}
