#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <memory.h>
#include <math.h>

#include "project.h"

#include <f2c.h>

#ifndef MIN
#define MIN(x, y) ((x) < (y) ? (x) : (y))
#endif

static void print_matrix(int m, int n, double *A)
{
  int i, j;

  for(i = 0; i < m; i++) {
    for(j = 0; j < n; j++) {
      printf("%7.4g ", A[i + m*j]);
    }
    fputc('\n', stdout);
  }
}

int dgeqp3_(const integer *m, const integer *n,
            double *A, const integer *lda, integer *jpvt, double *tau,
            double *work, integer *lwork, integer *info);

int dormqr_(const char *side, const char *trans,
            const integer *m, const integer *n, const integer *k,
            double *A, const integer *lda, const double *tau,
            double *c, const integer *ldc,
            double *work, integer *lwork, integer *info);

void project_nullspace(int in_m, int in_n, const double *in_A, double *x)
{
  double *A, *work, *tau, worksize;
  integer i, lwork, lwork2, info, m, n, k, nreflect, lda, ldc, *jpvt;
  char side, trans;

  ////// Compute QR = A'
  lda = m = (integer)in_n;
  n = (integer)in_m;
  nreflect = MIN(m, n);

  // Allocate input/output
  A = (double *)malloc(sizeof(double) * (m * n + nreflect) + sizeof(integer) * n);
  tau = A + m*n;
  jpvt = (integer *)(tau + nreflect);
  memcpy(A, in_A, sizeof(double) * m * n);

  // Get workspace size
  lwork = -1;
  dgeqp3_(&m, &n, A, &lda, jpvt, tau, &worksize, &lwork, &info);
  assert(info == 0);

  lwork = (integer)worksize;
  work = (double *)malloc(sizeof(double) * lwork);

  // Run QR decomposition
  dgeqp3_(&m, &n, A, &lda, jpvt, tau, work, &lwork, &info);
  assert(info == 0);

  ////// Calculate x = Q' * x

  side = 'L';
  trans = 'T';
  k = 1;

  // Get workspace size
  lwork2 = -1;
  dormqr_(&side, &trans, &m, &k, &nreflect,
          A, &lda, tau, x, &m,
          &worksize, &lwork2, &info);
  assert(info == 0);

  if(lwork2 > lwork) {
    lwork = lwork2;
    work = realloc(work, sizeof(double) * lwork);
  }

  // Run multiplication
  dormqr_(&side, &trans, &m, &k, &nreflect,
          A, &lda, tau, x, &m,
          work, &lwork, &info);
  assert(info == 0);

  /////// Zero out image columns of x
  for(i = 0; i < nreflect; i++) {
    if(fabs(A[i + n*i]) > 1e-6) {
      x[i] = 0;
    }
  }

  /////// Calculate x = Q*x
  trans = 'N';

  // Get workspace size
  lwork2 = -1;
  dormqr_(&side, &trans, &m, &k, &nreflect,
          A, &lda, tau, x, &m,
          &worksize, &lwork2, &info);
  assert(info == 0);

  if(lwork2 > lwork) {
    lwork = lwork2;
    work = realloc(work, sizeof(double) * lwork);
  }

  // Run multiplication
  dormqr_(&side, &trans, &m, &k, &nreflect,
          A, &lda, tau, x, &m,
          work, &lwork, &info);
  assert(info == 0);

  // Cleanup
  free(A);
  free(work);
}
