#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "project.h"

double frand()
{
  return rand() / (double)RAND_MAX;
}

void print_matrix(int m, int n, double *A)
{
  int i, j;

  for(i = 0; i < m; i++) {
    for(j = 0; j < n; j++) {
      printf("%7.4g ", A[n*i + j]);
    }
    fputc('\n', stdout);
  }
}

void mulvec(int m, int n, const double *A, const double *x, double *Ax)
{
  int i, j;

  for(i = 0; i < m; i++) {
    Ax[i] = 0;
    for(j = 0; j < n; j++) {
      Ax[i] += A[n*i + j] * x[j];
    }
  }
}

double dotprod(int n, const double *x, const double *y)
{
  double S;
  int i;

  S = 0;
  for(i = 0; i < n; i++) {
    S += x[i] * y[i];
  }

  return S;
}

int main(int argc, char **argv)
{
  unsigned int seed;
  int i, m, n;
  double *A, *x, *Ax, *xproj, *xdiff;

  seed = (unsigned int)time(NULL);
  printf("seed = %u\n", seed);
  srand(seed);

  m = 2;
  n = 4;

  // Allocate
  A = (double *)malloc(sizeof(double) * (m + 4) * n);
  x = A + m * n;
  Ax = x + n;
  xproj = Ax + n;
  xdiff = xproj + n;

  // Randomize input
  for(i = 0; i < m * n; i++) {
    A[i] = frand();
  }
  for(i = 0; i < n; i++) {
    xproj[i] = x[i] = frand();
  }

  printf("A = [\n");
  print_matrix(m, n, A);
  printf("]\nx = [\n");
  print_matrix(n, 1, x);
  
  mulvec(m, n, A, x, Ax);
  printf("]\nAx = \n");
  print_matrix(m, 1, Ax);
  
  project_nullspace(m, n, A, xproj);

  printf("\n\nproj(x) = \n");
  print_matrix(n, 1, xproj);

  mulvec(m, n, A, xproj, Ax);
  printf("\nA * proj(x) = \n");
  print_matrix(m, 1, Ax);

  for(i = 0; i < n; i++) {
    xdiff[i] = x[i] - xproj[i];
  }
  printf("\nproj(x) * (x - proj(x)) = %g\n", dotprod(n, xproj, xdiff));

  // Cleanup
  free(A);

  return 0;
}
