#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <f2c.h>
#include <math.h>

#include "rcp.h"
#include "project.h"
#include "sample.h"

static void samplep(unsigned long int *seed, int n, const double *X, double kappa, double *p)
{
  int i, j;
  double *mu, *sigma;
  sample_t *smp;

  mu = (double *)malloc(sizeof(double) * n * (n + 1));
  sigma = mu + n;
  for(i = 0; i < n * n; i++) {
    sigma[i] = (1 - kappa) * X[i];
  }

  for(i = 0; i < n; i++) {
    for(j = 0; j < n; j++) {
      sigma[n*i + j] += kappa;
    }
  }

  for(i = 0; i < n; i++) {
    mu[i] = 0;
  }

  smp = sample_init(*seed);
  sample_gaussian(smp, n, 1, p, mu, sigma);

  // Cleanup
  sample_end(smp);
  free(mu);

  // TODO, get proper seed
  *seed += 1;
}

void random_conic_pursuit(
  int n, double *X, int niters, double kappa,
  unsigned long int *seed,
  const sdp_problem *prob)
{
  int i, j, iter;
  double *p, *Y, alpha, beta;

  p = (double *)malloc(sizeof(double) * n * (n + 1));
  Y = p + n;

  for(iter = 0; iter < niters; iter++) {
    // Sample Y
    samplep(seed, n, X, kappa, p);
    for(j = 0; j < n; j++) {
      for(i = 0; i < n; i++) {
        Y[i + n*j] = p[i] * p[j];
      }
    }
    
    // Project Y onto nullspace of equality constraint matrix
    if(prob->nequal > 0) {
      // TODO: save QR decomposition
      project_nullspace(prob->nequal, n*n, prob->equal, Y);
    }

    // TODO: optimization

    // Update X
    if(alpha > 0) {
      for(i = 0; i < n*n; i++) {
        X[i] = alpha * Y[i] + beta * X[i];
      }
    }
  }

  // Cleanup
  free(p);
}
