#ifndef __RCP_H__
#define __RCP_H__

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {

  void *closure_data;

  double (*f)(void *closure_data, int n, const double *X);
  void (*df)(void *closure_data, int n, const double *X, double *dX);
  void (*ddf)(void *closure_data, int n, const double *X, double *ddX);

  int nconstraints;
  double (**g)(void *closure_data, int n, const double *X);
  void (**dg)(void *closure_data, int n, const double *X, double *d);

  /**
   * Number of linear equality constraints
   */
  int nequal;

  /**
   * Linear equality constraints, concatenated in order
   */
  double *equal;

} sdp_problem;

/**
 * @arg X On input, the initial matrix.  On output, the solution.  Column-major.
 */
void random_conic_pursuit(int n, double *X, int niters, double kappa,
                          unsigned long int *seed,
                          const sdp_problem *prob);

#ifdef __cplusplus
}
#endif

#endif // #ifndef __RCP_H__
